/*--------------------------------------------------------------------------*
 *  Copyright Unpublished Work of Novell, Inc. All Rights Reserved
 *
 *  THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
 *  PROPRIETARY AND TRADE SECRET INFORMATION OF NOVELL, INC.
 *  ACCESS TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES
 *  WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE
 *  OF THEIR ASSIGNMENTS AND (ii) ENTITIES OTHER THAN NOVELL
 *  WHO HAVE ENTERED INTO APPROPRIATE LICENSE AGREEMENTS.
 *  NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,
 *  DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED,
 *  CONDENSED, EXPANDED, COLLECTED, COMPILED, LINKED, RECAST,
 *  TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN CONSENT OF
 *  NOVELL.  ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION
 *  COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
 *--------------------------------------------------------------------------*/
/*****************************************************************************
 *
 * Title:			CMSM Configuration Parser
 *
 * Filename:		parser.c
 *
 * ODI Spec Ver:	1.11
 *
 * Description:		This module parses user supplied LAN driver parameters.
 *
 * Modification History:
 *
 * 07-03-96 TNL	Add code to bottom of CMSMParseDriverParameters to ensure
 *						MLIDCFG_CommandString does NOT get clobbered.
 *						SPD 130276.
 *
 * 10-15-96 MCD	Fixed code in CMSMParseDriverParameters in ifdef NWCACS to
 *              	get the correct offset to VirtualBoardLink
 *
 * 10-16-96 MSC	Fixed code at end of CMSMParseDriverParameters to pick up
 *              	the correct adapterData ptr for a CHSM.
 *
 * 01-07-97 MPK	Added code in ParseParam to ensure that the user will
 *                Only be prompted for framtype, if there is no FRAME=
 *              	string on the command line.
 *
 * 02-04-97 MPK	Changed code in CMSMParseDriverParameters to key off
 *						of DPB_Reserved0 (DriverBoardPointer ASM spec) insteadk
 *						of DriverAdapterPointer for determination of existance of
 *						frame data space.  This combines with changes made in
 *						CMSMRegisterTSM.
 *
 * 02-04-97 MPK	Changed code in CMSMParseDriverParameters to zero out
 *						logicalName field when the parser is
 *						using the template, and not a FrameDataspace
 *						this will allow the NAME parseing to update the
 *						template when executing.  This code will only
 *						affect those HSMs calling the parser before
 *						calling TSMRegisterHSM.
 *
 * 02-14-97 AYD	Added a new API CMSMParseSingleParameter which will unload some of
 *						the functions of an API CMSMParseDriverParameters
 *			
 * 02-19-97 MPK	Changed code in this module to take advantage of the new APIs
 *						CMSMParseSingleParameter (see AYD above), and CMSMInitParser
 *						(Also added by me at this time).
 *						These changes will simplify the parsing process,
 *						and should provide greater flexibility in the future.
 *
 * 02-20-97 MPK	Changed the name of CMSMParseParameter to CMSMParseSingleParameter
 *						per group decission.
 *
 * 02-25-97 WTT	Changed CMSMParseSingleParameter to match final spec.
 *						The parameter DRIVER_PARM_BLOCK *hsmParamBlock was not
 *						used or needed and was removed.
 *
 * 02-28-97 MPK	Changed the fix of 01-07-97 to be used when NWCACS is not defined.
 *
 * 05-29-97 KRA	In CMSMFindParameter checked the type parse type for numeric
 *						types.  If numeric, converted the number to a string using
 *						the conversion type.  Used the string containing the number
 *						rather than the number itself when calling CMSMGetParameter.
 *						SPD 153186.
 *
 * 					Added a case NO_PARAM_TYPE_SPECIFIED to ParseDriverOption().
 *						In this case allow a call to ParseParseString().
 *						This allows paramters that are NOT a KEYWORDPARAM, RANGEPARAM,
 *						ENUMPARAM, or a STRINGPARM to be able to be parsed for.  The
 *						default case still returns ODISTAT_FAIL.
 *						SPD 152977, 153720, and 1573700.
 *
 *						If an input paramter is out of range for a RANGEPARAM,
 *						prompting should continue until an input is received that
 *						falls with in the range.  The	ForcePrompt variable is used
 *						to cause prompting.  Fixed CMSMGetParameter() to look at
 *						the ForcePrompt variable and prompt rather than returning
 *						ODISTAT_ITEM_NOT_PRESENT.  Then set ForcePrompt to FALSE
 *						right after the call to CMSMGetParameter().  This solves the
 *						problem for both OPTIONALPARAM and REQUIREDPARAM.
 *						SPD 151817.
 *
 *						Initialized defaultString[0] to NULL in CMSMFindEnumParameter
 *						and CMSMFindRangeParameter.
 *						SPD 147898.
 *
 *						In CMSMGetParameter(), when checking the value of the
 *						parameter string, the pointer was being compared to null
 *						rather than what the pointer pointed to.  Changed if (s==NULL)
 *						to if (*s == NULL).
 *						SPD 158861.
 *
 *                In ParseParam, created paramBuf to be passed into all the
 *                CMSMFindParameter, CMSMFindRageParameter,
 *                and CMSMFindEnumParameter calls rather than using the
 *                option->String field.  Then if the pramater functions are
 *                successful, then do a CMSMsscanf from paramBuf to
 *                option->String.  This will fix the %5s, %[xxx] and %[^xxx]
 *                types of strings.
 *                SPD 153399, 153792, and 153807.
 *
 * 06-02-97 KRA   Made the ENUMPARAM | STRINGPARAM the same case as the
 *                ENUMPARAM.  The two cases had the same code.  Checked that
 *                the conversion spec length was not longer that the buffer.
 *
 *                In ParseParam added the ability to handle conversion string
 *                such as %5s%c%[abc]%s%[^def].  In CMSMGetParameter, summed
 *                the length of all the strings that belong to the set of
 *                conversion specifiers to know where the NULL
 *                goes in the returned param.  This puts mutliple string
 *                belonging to the same parameter in the same param buffer
 *                to be later scan converted based on the conversion specifier
 *                set.
 *                SPD 153782.
 *
 * 06-04-97 KRA	Removed the %S and %C as valid conversion specifiers.  %s and
 *						%c are still valid.
 *
 * 06-05-97 KRA	In CMSMGetParameter(), if the keyword is missing and is not
 *						required, return ODISTAT_ITEM_NOT_PRESENT.
 *						SPD 159457.
 *
 * 07-09-97 JRT   Modified ParseParam to not scan format string for conversion
 *                specifiers if the KEYWORDPARAM flag is set. SPD 161272
 *
 * 07-14-97 KRA	Checked the return code from ParseParseString in
 *						ParseDriverOption.  Changed CMSM_PARSER_BUFFERS16_TXTMSG to
 *						CMSM_PARASER_BUFFERS16_EQUAL_TXTMSG.
 *						SPD# 161702
 *
 * 07-16-97 KRA	In ParseParameter if we were parsing for FRAME then check to
 *						see if LSB is also on the command line.  If it is
 *						concatinate it to the frame type.
 *						SPD# 161370
 *
 * 07-22-97 KRA   Set gBuffersBelow16 to 0 in CMSMInitParser.
 *						SPD 162165.
 *
 * 08-12-97 JRT   Conditional compile out adding LSB to FRAME string for
 *          server.  SPD 163467
 *
 * 08-18-97 KRA   Removed dead code in ParseStdParam for printing the
 *						CMSM_PARSER_BOARD_NAME_MSG.  In CMSMGetParameter added the
 *						printing of the message CMSM_PARSER_KEYWORD_VALUE_MISSING_MSG
 *						if the value for a keyword is missing.
 *						SPD 164009.
 *
 * 08-19-97 KRA   Removed the conditional compile for adding LSB to the FRAME
 *						string.  The TSMs will now handle looking for LSB.
 *						SPD 164110.
 *
 * 01-22-98 KRA	Merged code between NT and Client32.
 *
 * 02-10-98 KRA	Fixed CMSMGetParameter to put a null in the param var for
 *						all parse types.  Fixes problem when parsing for a string
 *						that is an enum type, the string is on the command line
 *						and the parameter is in the middle of the command line.
 *						Without the fix the returned parameter contains not only
 *						the parsed string but whatever else comes on the command
 *						line after the the parsed for string.
 *						SPD 178852.
 *
 * 08-24-98 KRA	Fixed CMSMParseSingleParameter to check that the driver
 *						option's ParseString is non null.  If null CMSMstrlen will
 *						cause the server to abend.
 *						SPD 201862.
 *
 * 10-20-98 KRA	Fixed the parsing of enumerated string parameters so that the
 *						parser can distinguish between parameters that were prompted
 *						for and pramaters that were supplied on the command line.
 *						Added a new parameter, paramFlags, to CMSMGetParameter().
 *						Added #defines for the paramFalgs: DEFAULT_PARAMETER_FLAGS,
 *						PARAMETER_WAS_PROMPTED_FOR, and PARAMETER_DEFAULT_WAS_USED.
 *						Upon successful return the paramFlags will indicate if the
 *						parameter was prompted for or if the default was used.  This
 *						aids CMSMFindEnumParameter() for the string case to know
 *						if the returned parameter is the string given on the command
 *						line or if it is the parameter index obtained due to
 *						prompting.  This allows CMSMFindEnumParameter() to correctly
 *						handle parmeters that are enumerated strings.  Both strings
 *						representing numbers and strings that are just text may be
 *						used as options for the same parameter.
 *						SPD 214178.
 *
 *						Fixed CMSMFindEnumParameter() to upper case the options
 *						string before doing a string compare on the parameter
 *						returned from CMSMGetParameter().  The returned is always
 *						in upper case.  With out this it was possible for the parser
 *						to continually loop looking for the parameter that was
 *						supplied on the command line but failed to match due to case
 *						differences.
 *
 *						Fixed CMSMFindEnumParameter() so that if a string parameter
 *						on the command line did not match one in the supplied option
 *						list, the parameter would be prompted for.
 *
 *						Fixed CMSMFindEnumParameter() so that if a string parameter
 *						was prompted for and the user entered an invalied index
 *						the user was prompted again until a valid index was entered.
 *
 * 11-16-98 KRA	Check the flags field of the screen handle in
 *						CMSMPromptUserForInput.  If the keyboard is in use by another
 *						module, most likely install, get the screen for an alternate
 *						system console screen.  If it is ok to use that screen, make
 *						it the active screen an go ahead an prompt the user.
 *						Otherwise fail the prompt so as not to abend.  Added
 *						prototypes for GetSystemConsoleScreen and ActivateScreen.
 *						SPD 215925.
 *						
 *
 ****************************************************************************/
#ifdef OS_NT
/*===[ Include files specific to NT ]======================================*/
#include <ntos.h>
#define	INCL_RESMGR_SERVICE_MACROS			//define this for RESMGR.H file
#include <resmgr.h>
#endif

/*===[ Include files specific to current platform ]======================*/

#include <portable.h>

/*===[ Generic HSM Include files ]===========================*/
#include <odi.h>
#include	<odi_nbi.h>
#include <parser.h>
#include <cmsm.h>
#include	<msmstruc.h>
#include	<splat.h>
#ifdef OS_NT
#include <cmsmglu.h>
#include <cmsmgbls.h>
#include <odintmsg.h>
#include <msmstrng.h>
#else
#include <generic.h>
#endif
#include <cmsm.txt>
#ifdef OS_NIOS
#include <niosvid.h>
#endif

#ifndef NULL
#define NULL 0L
#endif

/*===[ Include files specific to this module ]===========================*/

#include <cmsmstr.h>

/*===[ External prototypes ]=============================================*/

#ifndef OS_NT
extern AsmCustomKeywordRoutine();

#ifdef SVR
UINT32   ValidateAddressRange(
            void     *pointer,
            UINT32   length,
            UINT32   accessType);

SCREEN_HANDLE *GetSystemConsoleScreen(void);

UINT32 ActivateScreen(SCREEN_HANDLE *);

void SetInputToOutputCursorPosition(SCREEN_HANDLE   *screenID);

void EnableInputCursor(SCREEN_HANDLE   *screenID);

void DisableInputCursor(SCREEN_HANDLE   *screenID);

int InputFromKeyboard(
            SCREEN_HANDLE  *screenID,
				char           *AllowedCharSet,
				UINT16	      bufferLength,
				UINT16	      editWidth,
				char		      *buffer,
				UINT32	      linesToProtect,
				BOOLEAN	      hasDefault,
				char		      *defaultString);

UINT32      ParseDriverParameters(
               IO_CONFIG                                 *IOConfig,
               CONFIG_TABLE                              *configuration,
               struct AdapterOptionDefinitionStructure   *adapterOptions,
               void                                      *configLimits,
               MEON                                      *frameTypeDescription[],
               UINT32                                    needBitMap,
               MEON                                      *commandLine,
               struct ScreenStruct                       *screenID);
#endif

#ifdef NWCACS

extern UINT32        CACSNumberFrames(      FRAME_DATA *framePtr  ) ;
extern void          CACSMediaNodeOverride( FRAME_DATA *frameData,
                                            UINT32      tail ) ;
extern ADAPTER_DATA *CACSGetAdapterDataPtr( FRAME_DATA *framePtr  ) ;

#endif
#endif

/*===[ Manifest constants ]==============================================*/


/* Variable equates (for convenience and/or clarity) */
#ifndef OS_NT
#define MULTIPLE_OF_8 0xfffffff8
#endif

/* Parameter flags used to inform the caller of CMSMGetParameter about the */
/* parameter that was returned if the call was successufl. */
#define DEFAULT_PARAMETER_FLAGS		0x00000000
#define PARAMETER_WAS_PROMPTED_FOR	0x00000001
#define PARAMETER_DEFAULT_WAS_USED	0x00000002


/*===[ Type definitions ]==================================================*/


/*===[ Global variables ]==================================================*/
MODULE_TRACK   *LoadedModules;
MEON_STRING		*CommandLine;
UINT32 			gBelow16 = 0;
BOOLEAN			parserInitialized = FALSE;	
#ifdef OS_NT
SCREEN_HANDLE  *CMSMScreenID = NULL;
#else
BOOLEAN        AskIfNewFrame = FALSE;
BOOLEAN        NewFrame = FALSE;
BOOLEAN			ForcePrompt = FALSE;
BOOLEAN			TSMParseFlag = FALSE;
CONFIG_TABLE   *CurrentConfigTable;
UINT32			gBuffersBelow16 = 0;
UINT32			MatchBoardNum = -1;

DRIVER_OPTION NameOption =
			{ NULL, NULL, 0, 0, 0, NAMEPARAM, OPTIONALPARAM | STRINGPARAM, };

DRIVER_OPTION BusOption =
			{ NULL, NULL, 0, 0, 0, BUSPARAM, OPTIONALPARAM | STRINGPARAM, };

DRIVER_OPTION RetriesOption =
			{ NULL, NULL, 0, 0, 0, RETRIESPARAM, OPTIONALPARAM | STRINGPARAM, };

DRIVER_OPTION Below16Option =
			{ NULL, NULL, 0, 0, 0, BELOW16PARAM, OPTIONALPARAM | KEYWORDPARAM, };

DRIVER_OPTION Buffers16Option =
			{ NULL, NULL, 0, 0, 0, BUFFERS16PARAM, OPTIONALPARAM | STRINGPARAM, };
#endif
/*===[ Function prototypes ]=============================================== */

ODISTAT ParseParameter(
					CONFIG_TABLE	*configTable,
					struct _DRIVER_OPTION_	*OriginalDriverOption);

ODISTAT ParseDriverOption(
					struct _DRIVER_OPTION_	*driverOption,
					UINT32		*ParseType,
					MEON_STRING **Keyword,
					MEON_STRING **CnvSpec,
					MEON_STRING **SCnvSpec);

ODISTAT	ParseParseString(
	      		MEON_STRING *ParseString,
					UINT32		*ParseType,
					MEON_STRING **Keyword,
					MEON_STRING **CnvSpec,
					MEON_STRING **SCnvSpec);

ODISTAT ParseStdParam(
					CONFIG_TABLE	*configTable,
					struct _DRIVER_OPTION_	*originalDriverOption,
					UINT32	ParseType,
					MEON_STRING *Keyword,
					MEON_STRING *CnvSpec,
					MEON_STRING *SCnvSpec);

ODISTAT ParseParam(
					struct _DRIVER_OPTION_	*driverOption,
					UINT32	ParseType,
					MEON_STRING *Keyword,
					MEON_STRING *CnvSpec,
					MEON_STRING *SCnvSpec);

ODISTAT	CMSMGetParameter(
					MEON_STRING			*Keyword,
					SCREEN_HANDLE     *screenID,
					MEON_STRING			*CommandLine,
               BOOLEAN           needToPrompt,
					MEON_STRING			*prompt,
					MEON_STRING			*Default,
					UINT16				Flags,
					UINT16				ConversionSpecifierCount,
					MEON_STRING			*param,
					UINT32				*paramFlags);

ODISTAT CMSMFindParameter(
					MEON_STRING			*Keyword,
					SCREEN_HANDLE     *screenID,
					MEON_STRING			*CommandLine,
					void					*defaultValue,
					MEON_STRING			*ConversionSpecifier,
					UINT16				Flags,
					UINT16				ParameterType,
					UINT16				ConversionSpecifierCount,
					MEON_STRING			*param);

ODISTAT CMSMFindEnumParameter(
					MEON_STRING       *Keyword,
					SCREEN_HANDLE     *screenID,
					MEON_STRING			*CommandLine,
					void              *defaultValue,
					MEON_STRING       *ConversionSpecifier,
					UINT16            Flags,
					UINT16				ParameterType,
					DRIVER_OPTION     *OptionPtr,
					MEON_STRING       *param);

ODISTAT CMSMFindRangeParameter(
					MEON_STRING			*Keyword,
					SCREEN_HANDLE     *screenID,
					MEON_STRING			*CommandLine,
					void					*defaultValue,
					MEON_STRING			*ConversionSpecifier,
					UINT16				Flags,
					UINT16				ParameterType,
					int					Min,
					int					Max,
					MEON_STRING			*param);

MEON * CMSMParseStringForKeyword(MEON *searchString, MEON *keyword);

#ifdef OS_NT
#define CMSMIsValidPtr(pointer) MmIsAddressValid(pointer)
#else
BOOLEAN CMSMIsValidPtr(void		*Pointer);

ODISTAT CMSMPromptUserForInput(
					SCREEN_HANDLE     *screenID,
					MEON_STRING 		*AllowedCharSet,
					MEON_STRING 		*prompt,
					MEON_STRING 		*defaultValueString,
					MEON_STRING 		*param,
					UINT32				bufferSize);

ODISTAT CMSMCheckRange(
		UINT32 Base0,
		UINT32 Range0,
		UINT32 Base1,
		UINT32 Range1);

ODISTAT  CMSMCheckValue(
		UINT8 Value0,
		UINT8 Value1);

ODISTAT CMSMCheckIOPort(
		UINT32 IOPort,
		UINT32 IOLength,
		UINT8  ShareFlag);

ODISTAT  CMSMCheckMemoryDecode(
		UINT32   MemoryAddress,
		UINT32   MemoryLength,
		UINT8    ShareFlag);

ODISTAT CMSMCheckInterrupt(
		UINT32   InterruptNumber,
		UINT8    ShareFlag);

ODISTAT CMSMCheckSlot(
		UINT32 SlotNumber);

ODISTAT  CMSMCheckDMA(
		UINT32   DMALevel,
		UINT8    ShareFlag);

ODISTAT  CMSMCheckForHardwareConflict(
            UINT32         value,
  				UINT16	      ParameterType);

UINT32  	CMSMCheckDuplicateEntry(void *value, UINT16 parameterType);
#endif

/*======================================================================
  Procedure Name:	CMSMParseDriverParameters

  Arguments:	
					hsmParmBlock      -	Ptr to The HSM Parameter Block.
					driverOption 		-	Ptr to a linked list of driver option
												structures; one for each possible parameter.

  Returns:		ODISTAT

  Abstract:		This function will parse for each parameter described by a
					driverOption structure returning the value found in the
					driverOption structure and, if the parameter is a standard
					parameter, in the configTable.

  Notes:			CMSMInitParser and C<TSM>RegisterHSM must previously have
					been called.

=======================================================================*/
ODISTAT _cdecl
CMSMParseDriverParameters(
					DRIVER_PARM_BLOCK			*hsmParmBlock,
					struct _DRIVER_OPTION_	*driverOption)

{
	CONFIG_TABLE				*configTable;
	struct _DRIVER_OPTION_	*curOption;
	ODISTAT						status = ODISTAT_SUCCESSFUL;
   UINT32                  i;
	UINT32						numOptions;
#ifdef OS_NT
	DRIVER_OPTION NameOption =
		{ NULL, NULL, 0, 0, 0, NAMEPARAM, OPTIONALPARAM | STRINGPARAM, };

	DRIVER_OPTION RetriesOption =
			{ NULL, NULL, 0, 0, 0, RETRIESPARAM, OPTIONALPARAM | STRINGPARAM, };

	DRIVER_OPTION Below16Option =
			{ NULL, NULL, 0, 0, 0, BELOW16PARAM, OPTIONALPARAM | KEYWORDPARAM, };

	DRIVER_OPTION Buffers16Option =
			{ NULL, NULL, 0, 0, 0, BUFFERS16PARAM, OPTIONALPARAM | STRINGPARAM, };

	DRIVER_OPTION SlotOption =
         { NULL, NULL, 0, 0, 2, SLOTPARAM, OPTIONALPARAM | DEFAULTPRESENT, };
#else
	ODISTAT						promptStatus = ODISTAT_SUCCESSFUL;
	FRAME_DATA              *framePtr,
									*instanceBoardArray[MAX_PHYSICAL_BOARDS];
	UINT32                  frameIndex,instanceNum = 0,
									instanceCount = 0;
	MEON_STRING             *whichInstancePrompt, temp[10], *instanceString;
	ADAPTER_DATA            *adapterData;
	MEON_STRING					yesNoBuff[3];
	MEON_STRING					temp_LogicalName[18];	
#endif


	/* Check to see that the parser has been initialized by calling */
	/* CMSMInitParaser. */
	if (parserInitialized == FALSE)
	{
		return ODISTAT_FAIL;
	}
	/* Now DE-Initialize the parser. */
	parserInitialized = FALSE;	

	/* Check to ensure that there is a valid config table/frame date present. */
	/* CMSMRegisterTSM will point DPB_Reserved0 at the config table. */
   if (hsmParmBlock->DPB_Reserved0 == NULL)
	{
		return ODISTAT_FAIL;
	}
	/* We have a Frame Space setup the Config pointer. */
   configTable = (CONFIG_TABLE *)hsmParmBlock->DPB_Reserved0;

#ifndef OS_NT
   CurrentConfigTable = configTable;

   /* AskIfNewFrame is used to see if we need to prompt the user. */
   /* Default to not prompt. */
   AskIfNewFrame = FALSE;

   /* NewFrame is set TRUE if this is a new Frame for loaded board. */
	/* Default is not new Frame. */
	if (configTable->MLIDCFG_FrameID == 0)
	   NewFrame = FALSE;

	/* Lets Check to see if we are already Loaded. */
	/* Clear out the instance list. */
	for (i = 0; i < MAX_PHYSICAL_BOARDS; ++i)
		instanceBoardArray[i] = NULL;

	for (i = 0; i < LSLConfigTableptr->LMaxNumberOfBoards; ++i)
	{
#ifndef NWCACS
		framePtr = CMSMVirtualBoardArray[i];
		if ( framePtr )
		{
			if ( CMSMstrcmp(
							framePtr->ConfigTable.MLIDCFG_ShortName,
							configTable->MLIDCFG_ShortName) == 0 )
			{
            /* We need to see if any frames are unused. */

         	adapterData = (ADAPTER_DATA *)framePtr->DriverData - 1;
         	for (frameIndex = 0;
            	frameIndex < framePtr->MPBlock->MediaConfigLimits->NumberFrames;
	             ++frameIndex)
    	    	{
        	   	if (adapterData->VirtualBoardLink[frameIndex] == NULL)
            	      break;
	         }

    	      if ( (frameIndex < framePtr->MPBlock->MediaConfigLimits->NumberFrames) )
        	  	{
   					/* We are loaded set AskIfNewFrame TRUE. */
	   			AskIfNewFrame = TRUE;
			   	instanceBoardArray[instanceCount] = framePtr;
				  	if (framePtr->ConfigTable.MLIDCFG_BoardInstance > instanceNum)
        	      {
				   	instanceNum = framePtr->ConfigTable.MLIDCFG_BoardInstance;
              	  ++instanceCount;
	            }
    	       }
			}
		}
#else		  /* NWCACS Code follows */
		framePtr = CMSMVirtualBoardArray[i];
		if (framePtr)
		{
			if (CMSMstrcmp(
							framePtr->ConfigTable.MLIDCFG_ShortName,
							configTable->MLIDCFG_ShortName) == 0)
			{
            /* We need to see if any frames are unused. */
		         adapterData = (ADAPTER_DATA *)CACSGetAdapterDataPtr(framePtr) - 1;
            	for (frameIndex = 0; frameIndex < CACSNumberFrames( framePtr );
                   frameIndex++ )
    	        {
                if (adapterData->VirtualBoardLink[frameIndex] == NULL)
            	      break;
	            }
              if ( (frameIndex < CACSNumberFrames( framePtr )) )
        	    {
   					/* We are loaded set AskIfNewFrame TRUE. */
	   				AskIfNewFrame = TRUE;
			   		instanceBoardArray[instanceCount] = framePtr;
				   	if (framePtr->ConfigTable.MLIDCFG_BoardInstance > instanceNum)
        	       {
					   	instanceNum = framePtr->ConfigTable.MLIDCFG_BoardInstance;
                	  ++instanceCount;
	               }
    	        }
			}
		}
#endif  /* Ifndef NWCACS */
	}

/*  If the AskIfNewFrame flag is true and we don't have a			*/
/* "FRAME=" on the command line then let's go ahead and prompt to see if	*/
/* this is new frame OR new adapter	*/

	if ((AskIfNewFrame) &&
		!(CMSMParseStringForKeyword(CommandLine, CMSM_PARSER_FRAME_TXTMSG)) &&
		!(configTable->MLIDCFG_FrameID & ASK_FRAME_DONE) )
 	{
	   status = CMSMPromptUserForInput(
                     CMSMScreenID,
	      				CMSM_PARSER_VALID_YES_NO_MSG,
   	      			CMSM_PARSER_LOAD_ANOTHER_FRAME_MSG,
	   	      		CMSM_PARSER_N_MSG,
		   	      	yesNoBuff,
							2);

		if (status == ODISTAT_FAIL)
		{
			CMSMPrintString(
					configTable,
					MSG_TYPE_INIT_ERROR,
					CMSM_PARSER_DRVPARM_ERROR_MSG,
					NULL,
					NULL);

			return(ODISTAT_FAIL);
		}

		configTable->MLIDCFG_FrameID |= ASK_FRAME_DONE;

  		if (yesNoBuff[0] == 'y' || yesNoBuff[0] == 'Y')
     	{
	      NewFrame = TRUE;
		}
      AskIfNewFrame = FALSE;
	}
#endif

	/* Parse for all parameters HSMs need not worry about such as */
	/* NAME, RETRIES, BELOW16, etc. */
	status = ParseParameter( configTable, &NameOption);
	if (!((status == ODISTAT_SUCCESSFUL) || (status == ODISTAT_ITEM_NOT_PRESENT)))
	{
#ifndef OS_NT
		CMSMPrintString(
				configTable,
				MSG_TYPE_INIT_ERROR,
				CMSM_PARSER_DRVPARM_ERROR_MSG,
				NULL,
				NULL);
#endif
		return status;
	}

#ifdef OS_NT
	status = ParseParameter( configTable, &SlotOption);
	if (!((status == ODISTAT_SUCCESSFUL) || (status == ODISTAT_ITEM_NOT_PRESENT)))
	{
		return status;
	}
#else
	status = ParseParameter(configTable, &BusOption);
	if (!((status == ODISTAT_SUCCESSFUL) || (status == ODISTAT_ITEM_NOT_PRESENT)))
	{
		CMSMPrintString(
				configTable,
				MSG_TYPE_INIT_ERROR,
				CMSM_PARSER_DRVPARM_ERROR_MSG,
				NULL,
				NULL);
		return status;
	}
#endif

	status = ParseParameter( configTable, &RetriesOption);
	if (!((status == ODISTAT_SUCCESSFUL) || (status == ODISTAT_ITEM_NOT_PRESENT)))
	{
#ifndef OS_NT
		CMSMPrintString(
				configTable,
				MSG_TYPE_INIT_ERROR,
				CMSM_PARSER_DRVPARM_ERROR_MSG,
				NULL,
				NULL);
#endif
		return status;
	}

	status = ParseParameter( configTable, &Below16Option);
	if (!((status == ODISTAT_SUCCESSFUL) || (status == ODISTAT_ITEM_NOT_PRESENT)))
	{
#ifndef OS_NT
		CMSMPrintString(
				configTable,
				MSG_TYPE_INIT_ERROR,
				CMSM_PARSER_DRVPARM_ERROR_MSG,
				NULL,
				NULL);
#endif
		return status;
	}

	status = ParseParameter( configTable, &Buffers16Option);
	if (!((status == ODISTAT_SUCCESSFUL) || (status == ODISTAT_ITEM_NOT_PRESENT)))
	{
#ifndef OS_NT
		CMSMPrintString(
				configTable,
				MSG_TYPE_INIT_ERROR,
				CMSM_PARSER_DRVPARM_ERROR_MSG,
				NULL,
				NULL);
#endif
		return status;
	}

	/* This is the main parse loop to parse for each driver option record */
	/* passed in by the HSM. */
	status = ODISTAT_SUCCESSFUL;
	for (curOption = driverOption, numOptions = 0;
		curOption != NULL; curOption = curOption->Link, ++numOptions)
	{

		status = ParseParameter( configTable, curOption);

		if (!((status == ODISTAT_SUCCESSFUL) ||
			(status == ODISTAT_ITEM_NOT_PRESENT) ||
			(status == ODISTAT_DUPLICATE_ENTRY)))
		{
#ifndef OS_NT
			CMSMPrintString(
					configTable,
					MSG_TYPE_INIT_ERROR,
					CMSM_PARSER_DRVPARM_ERROR_MSG,
					NULL,
					NULL);
#endif
			return status;
		}
	}

	/* Don't return ODISTAT_ITEM_NOT_PRESENT if there is more than one */
	/* parameter in the driverOption list. */
	if (numOptions > 1 || status == ODISTAT_DUPLICATE_ENTRY)
		status = ODISTAT_SUCCESSFUL;

#ifndef OS_NT
   if ((NewFrame == TRUE) && !(configTable->MLIDCFG_FrameID & CHECK_MULTICARD_DONE) )
   {
		/* Defualt to only one instance. */
		i = 1;

		/* Check to see if we have more than one board loaded. */
		if (instanceCount > 1)
		{
			/* We need to check which instance to load. */
			/* Lets build a prompt string. */
			whichInstancePrompt = CMSMInitAlloc(1024);
			if (whichInstancePrompt)
			{
				CMSMstrcpy(whichInstancePrompt, CMSM_PARSER_NEWLINE_MSG);
				for (i = 0; i < instanceCount ; ++i)
				{
					framePtr = instanceBoardArray[i];
					if (framePtr)
					{
						CMSMitoa(temp, i + 1);
						CMSMstrcat(whichInstancePrompt, temp);
						CMSMstrcat(whichInstancePrompt, CMSM_PARSER_PERIOD_MSG);
						CMSMstrcat(whichInstancePrompt, framePtr->ConfigTable.MLIDCFG_ShortName);
						CMSMstrcat(whichInstancePrompt, CMSM_PARSER_USING_MSG);

						if (framePtr->ConfigTable.MLIDCFG_Slot == 0xFFFF)
						{
							/* This is not slot based display the hardware settings. */
							if (framePtr->ConfigTable.MLIDCFG_IOPort0 != 0)
							{
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_IO_PORTS_MSG);
								CMSMxtoa(temp, framePtr->ConfigTable.MLIDCFG_IOPort0);
								CMSMstrcat(whichInstancePrompt, temp);
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_TO_MSG);
								CMSMxtoa(temp,
									framePtr->ConfigTable.MLIDCFG_IOPort0
									+ framePtr->ConfigTable.MLIDCFG_IORange0);
								CMSMstrcat(whichInstancePrompt, temp);
							}
							if (framePtr->ConfigTable.MLIDCFG_IOPort1 != 0)
							{
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_AND_MSG);
								CMSMxtoa(temp, framePtr->ConfigTable.MLIDCFG_IOPort1);
								CMSMstrcat(whichInstancePrompt, temp);
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_TO_MSG);
								CMSMxtoa(temp,
									framePtr->ConfigTable.MLIDCFG_IOPort1
									+ framePtr->ConfigTable.MLIDCFG_IORange1);
								CMSMstrcat(whichInstancePrompt, temp);
							}
							if (framePtr->ConfigTable.MLIDCFG_MemoryAddress0 != NULL)
							{
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_MEMORY_MSG);
								CMSMxtoa(temp, (UINT32)framePtr->ConfigTable.MLIDCFG_MemoryAddress0);
								CMSMstrcat(whichInstancePrompt, temp);
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_TO_MSG);
								CMSMxtoa(temp,
									(UINT32)framePtr->ConfigTable.MLIDCFG_MemoryAddress0
									+ framePtr->ConfigTable.MLIDCFG_MemorySize0);
								CMSMstrcat(whichInstancePrompt, temp);
							}
							if (framePtr->ConfigTable.MLIDCFG_MemoryAddress1 != NULL)
							{
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_AND_MSG);
								CMSMxtoa(temp, (UINT32)framePtr->ConfigTable.MLIDCFG_MemoryAddress1);
								CMSMstrcat(whichInstancePrompt, temp);
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_TO_MSG);
								CMSMxtoa(temp,
									(UINT32)framePtr->ConfigTable.MLIDCFG_MemoryAddress1
									+ framePtr->ConfigTable.MLIDCFG_MemorySize1);
								CMSMstrcat(whichInstancePrompt, temp);
							}
							if (framePtr->ConfigTable.MLIDCFG_Interrupt0 != 0xFF)
							{
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_INTERRUPT_MSG);
								CMSMxtoa(temp, framePtr->ConfigTable.MLIDCFG_Interrupt0);
								CMSMstrcat(whichInstancePrompt, temp);
							}
							if (framePtr->ConfigTable.MLIDCFG_Interrupt1 != 0xFF)
							{
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_AND_MSG);
								CMSMxtoa(temp, framePtr->ConfigTable.MLIDCFG_Interrupt1);
								CMSMstrcat(whichInstancePrompt, temp);
							}
							if (framePtr->ConfigTable.MLIDCFG_DMALine0 != 0xFF)
							{
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_DMA_CHANNEL_MSG);
								CMSMxtoa(temp, framePtr->ConfigTable.MLIDCFG_DMALine0);
								CMSMstrcat(whichInstancePrompt, temp);
							}
							if (framePtr->ConfigTable.MLIDCFG_DMALine1 != 0xFF)
							{
								CMSMstrcat(whichInstancePrompt, CMSM_PARSER_AND_MSG);
								CMSMxtoa(temp, framePtr->ConfigTable.MLIDCFG_DMALine1);
								CMSMstrcat(whichInstancePrompt, temp);
							}
							CMSMstrcat(whichInstancePrompt, CMSM_PARSER_NEWLINE_MSG);
						}
						else
						{
							/* We are slot based. Display slot. */
							CMSMstrcat(whichInstancePrompt, CMSM_PARSER_SLOT_MSG);
							CMSMitoa(temp, framePtr->ConfigTable.MLIDCFG_Slot);
							CMSMstrcat(whichInstancePrompt, temp);
							CMSMstrcat(whichInstancePrompt, CMSM_PARSER_NEWLINE_MSG);
						}
					}
				}
				/* Now lets prompt for which board to use. */
				CMSMstrcat(whichInstancePrompt, CMSM_PARSER_BOARD_TO_LOAD_MSG);

				instanceString = CMSM_PARSER_1_MSG;
				i = 0;
				while (i == 0 || i > instanceCount)
				{
					promptStatus = CMSMPromptUserForInput(
									CMSMScreenID,
									CMSM_PARSER_VALID_U_INTS_MSG,
									whichInstancePrompt,
									instanceString,
									temp,
									10);
					/* Check for valid response. */
					if (promptStatus == ODISTAT_SUCCESSFUL)
					{
						if (CMSMstrspn(temp, CMSM_PARSER_VALID_U_INTS_MSG))
						{
							i = CMSMatoi(temp);
						}
					}
					else
					{
						/* The prompting failed for some reason.  Free memory and */
						/* return. */
						CMSMFree(NULL, whichInstancePrompt);		
						return promptStatus;
					}

				}
				CMSMFree(NULL, whichInstancePrompt);		
			}
		  	configTable->MLIDCFG_FrameID |= CHECK_MULTICARD_DONE;
		}
      /* Copy the original configTable IO_CONFIG portion except for  */
		/* CommandString	*/

		configTable->MLIDCFG_Reserved2 = configTable->MLIDCFG_CommandString;
		CMSMmemcpy(&temp_LogicalName,configTable->MLIDCFG_LogicalName,18);

      *(IO_CONFIG *)(&configTable->MLIDCFG_DriverLink)
         = *(IO_CONFIG *)(&instanceBoardArray[i - 1]->ConfigTable.MLIDCFG_DriverLink);

		configTable->MLIDCFG_CommandString = configTable->MLIDCFG_Reserved2;
		configTable->MLIDCFG_Reserved2 = 0;
		CMSMmemcpy(configTable->MLIDCFG_LogicalName,&temp_LogicalName,18);

#ifndef NWCACS
		configTable->MLIDCFG_FrameTypeString = (MEON_STRING *)instanceBoardArray[i - 1]->DriverData;
#else		  /* NWCACS Code follows */
   adapterData = (ADAPTER_DATA *)CACSGetAdapterDataPtr(instanceBoardArray[i - 1]);
		configTable->MLIDCFG_FrameTypeString = (MEON_STRING *)adapterData;
#endif  /* Ifndef NWCACS */

		configTable->MLIDCFG_FrameID |= TSM_FLAG;
   }
#endif
	return status;
}


/*======================================================================
  Procedure Name:	CMSMParseCTSMParameters

  Arguments:	
					configTable       -	Ptr to The configTable
					driverOption 		-	Ptr to a linked list of driver option
												structures; one for each possible parameter.

  Returns:		ODISTAT

  Abstract:		This function will parse for each parameter described by a
					driverOption structure returning the value found in the
					driverOption structure and, if the parameter is a standard
					parameter, in the configTable.

  Notes:				

=======================================================================*/
ODISTAT _cdecl
CMSMParseCTSMParameters(
					CONFIG_TABLE   			*configTable,
					struct _DRIVER_OPTION_	*driverOption)
{
	struct _DRIVER_OPTION_	*curOption;
	ODISTAT						status = ODISTAT_SUCCESSFUL;

#ifndef OS_NT
	TSMParseFlag = TRUE;
#endif

   for (curOption = driverOption; curOption != NULL;
		curOption = curOption->Link)
	{
		status = ParseParameter(
					configTable,
					curOption);

#ifndef OS_NT
		TSMParseFlag = FALSE;
#endif
		if (status == ODISTAT_FAIL)
			return status;
	}
#ifndef OS_NT
	TSMParseFlag = FALSE;
#endif
	return status;
}


/*======================================================================
  Procedure Name:	ParseParameter

  Arguments:	
					configTable       	-	Ptr to Our config Table.
					driverOption 			-	Ptr to a driver option structure.

  Returns:		ODISTAT


  Abstract:		This function will parse for one parameter described by the
					driverOption structure returning the value found in the
					driverOption structure and, if the parameter is a standard
					parameter, in the configTable.

  Notes:				

========================================================================*/
ODISTAT ParseParameter(
					CONFIG_TABLE				*configTable,
					struct _DRIVER_OPTION_	*originalDriverOption)
{
	UINT32					ParseType;
	ODISTAT					status;
	MEON_STRING				SCnvSpecBuf[MAX_CONV_SPEC_LEN];
	MEON_STRING				*Keyword, *CnvSpec;
	MEON_STRING				*SCnvSpec = SCnvSpecBuf;
	struct _DRIVER_OPTION_	driverOption;

   /* Lets clear out the original String value so that it won't be used. */
	CMSMmemset(
               originalDriverOption->String,
               0,
               MAX_PARAM_LEN);

	driverOption = *originalDriverOption;

	switch (driverOption.Type)
	{
		case INTPARAM:
			driverOption.ParseString = CMSM_PARSER_INT_TXTMSG;
			break;
		case INT1PARAM:
			driverOption.ParseString = CMSM_PARSER_INT1_TXTMSG;
			break;
		case PORTPARAM:
			driverOption.ParseString = CMSM_PARSER_PORT_TXTMSG;
			break;
		case PORT1PARAM:
			driverOption.ParseString = CMSM_PARSER_PORT1_TXTMSG;
			break;
		case DMAPARAM:
			driverOption.ParseString = CMSM_PARSER_DMA_TXTMSG;
			break;
		case DMA1PARAM:
			driverOption.ParseString = CMSM_PARSER_DMA1_TXTMSG;
			break;
		case MEMPARAM:
			driverOption.ParseString = CMSM_PARSER_MEM_TXTMSG;
			break;
		case MEM1PARAM:
			driverOption.ParseString = CMSM_PARSER_MEM1_TXTMSG;
			break;
		case SLOTPARAM:
			driverOption.ParseString = CMSM_PARSER_SLOT_TXTMSG;
			break;
		case CHANNELPARAM:
			driverOption.ParseString = CMSM_PARSER_CHANNEL_TXTMSG;
			break;
		case RETRIESPARAM:
			driverOption.ParseString = CMSM_PARSER_RETRIES_TXTMSG;
			break;
		case BUFFERS16PARAM:
			driverOption.ParseString = CMSM_PARSER_BUFFERS16_EQUAL_TXTMSG;
			break;
		case BELOW16PARAM:
			driverOption.ParseString = CMSM_PARSER_BELOW16_TXTMSG;
			break;
		case NODEPARAM:
			driverOption.ParseString = CMSM_PARSER_NODE_EQUAL_TXTMSG;
			break;
		case FRAMEPARAM:
			break;
		case BUSPARAM:
			driverOption.ParseString = CMSM_PARSER_BUS_EQUAL_TXTMSG;
			break;
		case NAMEPARAM:
			driverOption.ParseString = CMSM_PARSER_NAME_EQUAL_TXTMSG;
			break;
	}

	if ((status = ParseDriverOption(
					&driverOption,
					&ParseType,
					&Keyword,
					&CnvSpec,
					&SCnvSpec)) == ODISTAT_SUCCESSFUL)
	{
		if (driverOption.Type == CUSTOMPARAM)
		{
			status = ParseParam(
					&driverOption,
					ParseType,
					Keyword,
					CnvSpec,
					SCnvSpec);
		}
		else
		{
			status = ParseStdParam(
					configTable,
					&driverOption,
					ParseType,
					Keyword,
					CnvSpec,
					SCnvSpec);
		}
	}

#ifndef OS_NT
	if (driverOption.Flags & (ENUMPARAM | REQUIREDPARAM))
	{
		switch (driverOption.Type)
		{
			case PORTPARAM:
				if (configTable->MLIDCFG_IORange0 < originalDriverOption->Parameter1.Range)
		  			configTable->MLIDCFG_IORange0 =
					originalDriverOption->Parameter1.Range;
				break;
			case PORT1PARAM:
				if (configTable->MLIDCFG_IORange1 < originalDriverOption->Parameter1.Range)
		  			configTable->MLIDCFG_IORange1 =
					originalDriverOption->Parameter1.Range;
				break;
			case MEMPARAM:
				if (configTable->MLIDCFG_MemorySize0 < originalDriverOption->Parameter1.Range)
		  			configTable->MLIDCFG_MemorySize0 =
					originalDriverOption->Parameter1.Range;
				break;
			case MEM1PARAM:
				if (configTable->MLIDCFG_MemorySize1 < originalDriverOption->Parameter1.Range)
		  			configTable->MLIDCFG_MemorySize1 =
					originalDriverOption->Parameter1.Range;
				break;
		}
	}
#endif

	originalDriverOption->Parameter1.Max = driverOption.Parameter1.Max;

	CMSMmemcpy(originalDriverOption->String, driverOption.String, MAX_PARAM_LEN);

	return status;
}


/*=========================================================================
  Procedure Name:	ParseDriverOption

  Arguments:	
            driverOption 				-	Ptr to a driver option structure.
				ParseType					-	Ptr to a UINT32 returned indicating the
													type of the variable to be input.
														STRINGTYPE	= string
														INTTYPE			= 32 bit integer
														UINTTYPE		= UINT32
														XINTTYPE		= Hex UINT32
				Keyword						-	Ptr to null treminated keyword string.
				CnvSpec						-	Ptr to null terminated conversion specifier.
				SCnvSpec						-	Ptr to null terminated conversion specifier
													modified to %s type.

  Returns:	ODISTAT
				SUCCESSFUL					-	The option structure is valid.
				FAIL							-	The option structure is invalid.

  Abstract:	This routine checks a driverOption structure to see if it
				is valid.

  Notes:				

==========================================================================*/
ODISTAT ParseDriverOption(
					struct	_DRIVER_OPTION_	*driverOption,
					UINT32	*ParseType,
					MEON_STRING		**Keyword,
					MEON_STRING		**CnvSpec,
					MEON_STRING		**SCnvSpec)
{
	ODISTAT	status;

	status = ODISTAT_SUCCESSFUL;

	switch (driverOption->Type)
	{
		case INTPARAM:
		case INT1PARAM:
		case PORTPARAM:
		case PORT1PARAM:
		case DMAPARAM:
		case DMA1PARAM:
		case MEMPARAM:
		case MEM1PARAM:
		case SLOTPARAM:
			if (driverOption->Flags & ENUMPARAM)
			{
				if ( !CMSMIsValidPtr(driverOption->Parameter0.OptionPtr))
				{
					status = ODISTAT_FAIL;
				}
			}
			status = ParseParseString(
							driverOption->ParseString,
							ParseType,
							Keyword,
							CnvSpec,
							SCnvSpec);
			break;


		case BELOW16PARAM:
			*Keyword = driverOption->ParseString;
			*ParseType = STRINGTYPE;
			*CnvSpec = NULL;
			*SCnvSpec = NULL;
			break;

		case RETRIESPARAM:
		case BUFFERS16PARAM:
		case CHANNELPARAM:
		case NODEPARAM:
		case FRAMEPARAM:
		case BUSPARAM:
		case NAMEPARAM:
			status = ParseParseString(
							driverOption->ParseString,
							ParseType,
							Keyword,
							CnvSpec,
							SCnvSpec);
			break;

		case CUSTOMPARAM:
			switch (driverOption->Flags
				& (ENUMPARAM | RANGEPARAM | STRINGPARAM | KEYWORDPARAM))
			{
				case	ENUMPARAM:
				case	(ENUMPARAM | STRINGPARAM):
					if ( !CMSMIsValidPtr(driverOption->Parameter0.OptionPtr))
						status = ODISTAT_FAIL;

					if ( !CMSMIsValidPtr(driverOption->ParseString))
						status = ODISTAT_FAIL;

					status = ParseParseString(
									driverOption->ParseString,
									ParseType,
									Keyword,
									CnvSpec,
									SCnvSpec);

					if (*ParseType == STRINGTYPE
						&& (driverOption->Flags & DEFAULTPRESENT))
					{
						if ( !CMSMIsValidPtr(driverOption->Parameter2.StringDefault))
							status = ODISTAT_FAIL;
					}
					break;
					
				case	RANGEPARAM:
					if (driverOption->Parameter0.Min > driverOption->Parameter1.Max)
						status = ODISTAT_FAIL;

					if (driverOption->Parameter0.Min > driverOption->Parameter2.Default)
						status = ODISTAT_FAIL;

					if (driverOption->Parameter2.Default > driverOption->Parameter1.Max)
						status = ODISTAT_FAIL;

					status = ParseParseString(
							driverOption->ParseString,
							ParseType,
							Keyword,
							CnvSpec,
							SCnvSpec);

					if ((*ParseType & (INTTYPE | UINTTYPE | XINTTYPE)) == 0)
						status = ODISTAT_FAIL;
					break;

				case	STRINGPARAM:
					status = ParseParseString(
							driverOption->ParseString,
							ParseType,
							Keyword,
							CnvSpec,
							SCnvSpec);
					if (*ParseType != STRINGTYPE)
						status = ODISTAT_FAIL;
					break;

				case  KEYWORDPARAM:
					*Keyword = driverOption->ParseString;
					*ParseType = STRINGTYPE;
					*CnvSpec = NULL;
					*SCnvSpec = NULL;
					status = ODISTAT_SUCCESSFUL;
					break;

            case NO_PARAM_TYPE_SPECIFIED:
					status = ParseParseString(
							driverOption->ParseString,
							ParseType,
							Keyword,
							CnvSpec,
							SCnvSpec);
               break;

				default: /* Unknown DRIVER_OPTION "Flag" */
					status = ODISTAT_FAIL;
					break;
			}
			break;

		default:
			status = ODISTAT_FAIL;
			break;
	}
	return status;
}

/*======================================================================
  Procedure Name:	ParseParseString

  Arguments:	
							ParseString	- Ptr to a parse string as defined
																	in the CHSM spec.
							ParseType	- Ptr to a UINT32 returned indicating the
												type of the variable to be input.
														   STRINGTYPE	= string
														   INTTYPE = 32 bit integer
														   UINTTYPE = UINT32
														   XINTTYPE = Hex UINT32
							Keyword		- Ptr to null treminated keyword string.
							CnvSpec		- Ptr to null terminated conversion specifier.
							SCnvSpec		- Ptr to null terminated conversion specifier
												modified to %s type.

  Returns:		ODISTAT

  Abstract:			This routine will check the parse string and return
						whether or not it is legal and, if it is legal, what
						the type of the value to be parsed is, what the keyword
						to be searched for is, and what the conversion specifier
						is.

  Notes:				

=======================================================================*/
ODISTAT	ParseParseString(
	      		MEON_STRING *ParseString,
						UINT32	*ParseType,
						MEON_STRING **Keyword,
						MEON_STRING **CnvSpec,
						MEON_STRING **SCnvSpec)
{
	int	 	keywordlen, cnvspeclen, parsestringlen;
	MEON_STRING	*s, *t;
	static MEON_STRING keywordBuf[80];
	static MEON_STRING cnvSpecBuf[80];
   MEON_STRING *whiteSpace = CMSM_PARSER_WHITESPACE_SEPARATORS_MSG;
   MEON_STRING *separators = CMSM_PARSER_WHITESPACE_EQUAL_SEPARATORS_MSG;
	MEON_STRING *digits =	CMSM_PARSER_VALID_U_INTS_MSG;

	parsestringlen = CMSMstrlen (ParseString);

	s = ParseString;
	s += CMSMstrspn (s, whiteSpace);
	keywordlen = CMSMstrcspn(s, separators);
	CMSMstrncpy(keywordBuf, s, keywordlen);
	keywordBuf[keywordlen] = '\0';
	*Keyword = keywordBuf;
	s += keywordlen;

	if (s - ParseString > parsestringlen)
		return ODISTAT_FAIL;

	s += CMSMstrspn (s, separators);
	cnvspeclen = CMSMstrcspn (s, whiteSpace);

   /* Make sure there is enough space to store the conversion specifier. */
   if (cnvspeclen >= MAX_CONV_SPEC_LEN - 1)
   {
		return ODISTAT_FAIL;
   }

	CMSMstrncpy(cnvSpecBuf, s, cnvspeclen);
	cnvSpecBuf[cnvspeclen] = '\0';
	*CnvSpec = cnvSpecBuf;

	if (keywordlen == 0 || cnvspeclen == 0)
		return ODISTAT_FAIL;

	if (*s != '%')
		return ODISTAT_FAIL;

	t = s++;

	if (CMSMisdigit(*s))
	{
		s += CMSMstrspn (s, digits);
	}

	switch (*s)
	{
		case 'd':
		case 'D':
			*ParseType = INTTYPE;
			break;

		case 'u':
		case 'U':
			*ParseType = UINTTYPE;
			break;
		case 'x':
		case 'X':
			*ParseType = XINTTYPE;
			break;

		case 's':
			*ParseType = STRINGTYPE;
			break;

		case '[':
			s = CMSMstrchr (s, ']');
			if (s == NULL)
				return ODISTAT_FAIL;
			*ParseType = STRINGTYPE;

		case 'c':
			*ParseType = STRINGTYPE;
			break;

		default:
			*ParseType = BADTYPE;
			return ODISTAT_FAIL;
	}

	CMSMstrcpy(*SCnvSpec, *CnvSpec);
	if (*ParseType != STRINGTYPE)
	{
		t = *SCnvSpec + (s - t);
		*t = 's';
		++t;
		*t = '\0';
	}

	++s;
	while (*s != '\0')
	{
		if (CMSMstrcspn (s, whiteSpace) != 0)
		{
			if (*ParseType == STRINGTYPE)
			{
				if (*s != '%')
					return ODISTAT_FAIL;
				if (CMSMisdigit(*++s))
				{
					s += CMSMstrspn (s, digits);
				}
				if (*s != 's' && *s != 'c' && *s != '[')
					return ODISTAT_FAIL;
				if (*s == '[')
				{
					s += CMSMstrcspn (s, CMSM_PARSER_SQUARE_BRACKET_MSG);
				}
				++s;
			}
			else
				return ODISTAT_FAIL;
		}
		else
			return ODISTAT_SUCCESSFUL;
	}

	return ODISTAT_SUCCESSFUL;	
}



/*======================================================================
  Procedure Name:	ParseStdParam

  Arguments:			
							configTable       - Ptr to Our config Table.
             driverOption 			-	Ptr to a driver option structure.
							ParseType					- UINT32 indicating the type of the variable
																	to be input.
																		STRINGTYPE	= string
																		INTTYPE			= 32 bit integer
																		UINTTYPE		= UINT32
																		XINTTYPE		= Hex UINT32
							Keyword						- Ptr to null treminated keyword string.
							CnvSpec						- Ptr to null terminated conversion specifier.
							SCnvSpec					- Ptr to null terminated conversion specifier
																		modified to %s type.

  Returns:		ODISTAT


  Abstract:	This function will parse for one parameter described by the
							driverOption structure returning the value found in the
							driverOption structure and in the configTable.

  Notes:				

========================================================================*/
ODISTAT ParseStdParam(
					CONFIG_TABLE	*configTable,
					struct _DRIVER_OPTION_	*driverOption,
					UINT32	ParseType,
					MEON_STRING *Keyword,
					MEON_STRING *CnvSpec,
					MEON_STRING *SCnvSpec)

{
	ODISTAT		status;
#ifndef OS_NT
	UINT32		i;
	FRAME_DATA	*framePtr;
#endif

	status = ParseParam(
					driverOption,
					ParseType,
					Keyword,
					CnvSpec,
					SCnvSpec);

	if (status == ODISTAT_SUCCESSFUL || status == ODISTAT_DUPLICATE_ENTRY)
	{																			 	

		switch (driverOption->Type)
		{
			case INTPARAM:
				configTable->MLIDCFG_Interrupt0 = driverOption->Parameter1.Max;
				break;
			case INT1PARAM:
				configTable->MLIDCFG_Interrupt1 = driverOption->Parameter1.Max;
				break;
			case PORTPARAM:
				configTable->MLIDCFG_IOPort0 = driverOption->Parameter1.Max;
				break;
			case PORT1PARAM:
				configTable->MLIDCFG_IOPort1 = driverOption->Parameter1.Max;
				break;
			case DMAPARAM:
				configTable->MLIDCFG_DMALine0 = driverOption->Parameter1.Max;
				break;
			case DMA1PARAM:
				configTable->MLIDCFG_DMALine1 = driverOption->Parameter1.Max;
				break;
			case MEMPARAM:
				configTable->MLIDCFG_MemoryAddress0 = (void *)driverOption->Parameter1.Max;
				break;
			case MEM1PARAM:
				configTable->MLIDCFG_MemoryAddress1 = (void *)driverOption->Parameter1.Max;
				break;
			case SLOTPARAM:
				configTable->MLIDCFG_Slot = driverOption->Parameter1.Max;
				break;
			case CHANNELPARAM:
				configTable->MLIDCFG_ChannelNumber = driverOption->Parameter1.Max;
				break;
			case RETRIESPARAM:
				configTable->MLIDCFG_SendRetries = driverOption->Parameter1.Max;
				break;
			case BUFFERS16PARAM:
#ifndef OS_NT
				gBuffersBelow16 = MULTIPLE_OF_8 & driverOption->Parameter1.Max;
#endif
				break;
			case NODEPARAM: /* NODE and FRAME will be handled by the TSMs. */
				break;
			case FRAMEPARAM:
				break;
			case BUSPARAM:
#ifndef OS_NT
				GetBusTag((MEON_STRING *)&driverOption->String, &configTable->MLIDCFG_DBusTag);
#endif
				break;
			case NAMEPARAM:
				if (driverOption->String[0] != '\0')
				{
					if (CMSMstrlen(driverOption->String) > MAX_LOGICAL_NAME_SZ)
					{
#ifdef OS_NT
						CMSMLogEvent(
							MSG_TYPE_INIT_WARNING,
							BAD_BOARD_NAME,
							driverOption->String);
#else
						CMSMPrintString(configTable,
							MSG_TYPE_INIT_WARNING,
							CMSM_PARSER_BOARD_NAME_TO_LONG_MSG,
							driverOption->String,
							NULL);
#endif
						driverOption->String[MAX_LOGICAL_NAME_SZ] = '\0';
					}

#ifndef OS_NT
					/* Lets Check to see if this name is already used. */

					for (i = 0; i < LSLConfigTableptr->LMaxNumberOfBoards; ++i)
					{
						framePtr = CMSMVirtualBoardArray[i];
						if (framePtr)
						{
							if (CMSMstrcmp(
								framePtr->ConfigTable.MLIDCFG_LogicalName,
								driverOption->String) == 0)
							{
								CMSMPrintString(configTable,
									MSG_TYPE_INIT_WARNING,
									CMSM_PARSER_BOARD_NAME_USED_MSG,
									driverOption->String,
									NULL);
								break;
							}
						}
					}
#endif
					CMSMstrupr(driverOption->String);
					CMSMstrcpy(configTable->MLIDCFG_LogicalName,
						driverOption->String);
				}
				break;
			case BELOW16PARAM:
				gBelow16 = driverOption->Parameter1.Max;
				break;
		}
	}
	return status;
}

/*======================================================================
  Procedure Name:	ParseParam

  Arguments:			
				driverOption	-	Ptr to a driver option structure.
				ParseType		-	UINT32 indicating the type of the variable
											to be input.
											STRINGTYPE	= string
											INTTYPE	= 32 bit integer
											UINTTYPE	= UINT32
											XINTTYPE	= Hex UINT32
				Keyword			- Ptr to null treminated keyword string.
				CnvSpec			- Ptr to null terminated conversion specifier.
				SCnvSpec			- Ptr to null terminated conversion specifier
										modified to %s type.

  Returns:		ODISTAT


  Abstract:	This function will parse for one parameter described by the
							driverOption structure returning the value found in the
							driverOption structure.

  Notes:				

========================================================================*/
ODISTAT ParseParam(
					DRIVER_OPTION		*driverOption,
					UINT32				ParseType,
					MEON_STRING			*Keyword,
					MEON_STRING			*CnvSpec,
					MEON_STRING			*SCnvSpec)

{
	UINT32	      CnvSpecCnt = 0;
   UINT32         i;
   UINT32         j;
   UINT32         len;
   UINT32         fmtPos;
	MEON_STRING		*s;
	MEON_STRING		*whiteSpace = CMSM_PARSER_WHITESPACE_SEPARATORS_MSG;
	ODISTAT	      status;
   MEON				paramBuf[MAX_PARAM_LEN];
	MEON           fmt[MAX_CONV_SPEC_LEN];

   SCnvSpec = SCnvSpec;

	if (CnvSpec != NULL)
	{
		/* Ccount how many conversion specifier there are. */
		for (CnvSpecCnt = 0, s = CnvSpec; (s = CMSMstrchr(s, '%')) != NULL;
			++CnvSpecCnt, ++s);
	}

/* mpk 01-07-97 - if this is a frame, and the FRAME= is on the command line,
                  Then make sure we don't prompt again */
#ifndef NWCACS   /* for Client ONLY, MPK 02-28-97 */
	if (driverOption->Type == FRAMEPARAM)
		if (CMSMParseStringForKeyword(CommandLine, CMSM_PARSER_FRAME_TXTMSG))
			driverOption->Flags = STRINGPARAM | REQUIREDPARAM;
#endif

	paramBuf[0] = '\0';
	switch (driverOption->Flags
		& (ENUMPARAM | RANGEPARAM | STRINGPARAM | KEYWORDPARAM))
	{
		case	ENUMPARAM:

			status = CMSMFindEnumParameter(
                     Keyword,
                     CMSMScreenID,
                     CommandLine,
                     (void *)&driverOption->Parameter2,
                     CnvSpec,
							(UINT16)((driverOption->Flags & (REQUIREDPARAM | DEFAULTPRESENT)) | ParseType),
         	   		driverOption->Type,
         		   	driverOption,
                     paramBuf);
			break;

		case	RANGEPARAM:

			status = CMSMFindRangeParameter(
                     Keyword,
                     CMSMScreenID,
                     CommandLine,
                     (void *)&driverOption->Parameter2,
                     CnvSpec,
							(UINT16)((driverOption->Flags & (REQUIREDPARAM | DEFAULTPRESENT)) | ParseType),
         			   driverOption->Type,
            			driverOption->Parameter0.Min,
            			driverOption->Parameter1.Max,
                     paramBuf);

			if (status == ODISTAT_ITEM_NOT_PRESENT
				&& ((driverOption->Flags & (OPTIONALPARAM | DEFAULTPRESENT))
					== (OPTIONALPARAM | DEFAULTPRESENT)))
			{
				driverOption->Parameter1.Max = driverOption->Parameter2.Default;
			}
			break;

		case KEYWORDPARAM:
			if (CMSMParseStringForKeyword(CommandLine,
				driverOption->ParseString) != NULL)
			{
				driverOption->Parameter1.Max = 1;
				status = ODISTAT_SUCCESSFUL;
			}
			else
			{
				driverOption->Parameter1.Max = 0;
				status = ODISTAT_ITEM_NOT_PRESENT;
			}

			break;

		case	STRINGPARAM:
			status = CMSMFindParameter(
                     Keyword,
                     CMSMScreenID,
                     CommandLine,
                     (void *)driverOption->Parameter2.StringDefault,
                     CnvSpec,
							(UINT16)((driverOption->Flags & (REQUIREDPARAM | DEFAULTPRESENT)) | ParseType),
            			driverOption->Type,
							(UINT16)CnvSpecCnt,
                     paramBuf);
			break;

		default:
			status = CMSMFindParameter(
                     Keyword,
                     CMSMScreenID,
                     CommandLine,
                     (void *)&driverOption->Parameter2,
                     CnvSpec,
							(UINT16)((driverOption->Flags & (REQUIREDPARAM | DEFAULTPRESENT)) | ParseType),
            			driverOption->Type,
							(UINT16)CnvSpecCnt,
                     paramBuf);
			break;

	}

	/* Do not scan convert KEYWORDPARAMs. */
	if ( !(driverOption->Flags & KEYWORDPARAM) &&
		(status == ODISTAT_SUCCESSFUL || status == ODISTAT_DUPLICATE_ENTRY) )
	{

		if (ParseType & (INTTYPE | UINTTYPE | XINTTYPE))
		{
         CMSMstrcpy(driverOption->String, paramBuf);
			CMSMsscanf(driverOption->String, CnvSpec, &(driverOption->Parameter1.Max));
		}
      else /* STRINGTYPE */
      {
         /* Scan the string in case there is a %5s, %[xxx] or %[^xxx], etc. */
         s = paramBuf;
         s += CMSMstrspn(s, whiteSpace);

         /* Setup first format specifier. */
         fmt[0] = '%';
         fmtPos = 1;
         for (j = 1; CnvSpec[fmtPos] != '%' && CnvSpec[fmtPos] != '\0';
              j++, fmtPos++)
         {
            fmt[j] = CnvSpec[fmtPos];
         }
         fmt[j] = '\0';

         len = 0;
         for (i = 0; i < CnvSpecCnt; i++)
         {
            CMSMsscanf(s, fmt, &driverOption->String[len]);

            if (i < CnvSpecCnt - 1)
            {
               /* Skip over the token. */
               s += CMSMstrcspn(s, whiteSpace);

               /* Eat any whitespece between tokens. */
               s += CMSMstrspn(s, whiteSpace);

               /* Advance to the next conversion specifier. */
               fmtPos++;
               fmt[0] = '%';
               for (j = 1; CnvSpec[fmtPos] != '%' && CnvSpec[fmtPos] != '\0';
                  j++, fmtPos++)
               {
                  fmt[j] = CnvSpec[fmtPos];
               }
               fmt[j] = '\0';

               /* Advance driverOption->String to receive the next token. */
               len = CMSMstrlen(driverOption->String);
               driverOption->String[len] = ' ';
               len++;
            }
         }
      }
	}
	return status;
}


/*======================================================================

  Procedure Name:	CMSMGetParameter

  Arguments:	
					Keyword						-	Ptr to the keyword to be found
					screenID          		- Screen Handle for user I/O
					commandLine					-	Ptr to the String to search.
					prompt						-	Ptr to the Prompt string if needed.
					Default						-	Ptr to the default value this
														this is a ASCII string.
					Flags							-	0x0001 - set if the parameter is required
														- 0x0002 - set if there is a default
														- 0x0004 - set if the parameter is a string
														- 0x0008 - set if the parameter is an int
														- 0x0010 - set if the parameter is an unsigned int
														- 0x0020 - set if the parameter is an unsigned hexidecimal int
					ConversionSpecifierCount- count of the number of %s, %c, %[] spec's are
													  in ConversionSpecifier.
					param										- Output param, to be set to the paramter value found.
					paramFlags					-	Pointer to the returned paramer flags.
														On entry, paramFlags will be set to 0.
														On successful return, individual flags
														will be set to indicate information
														about the returned parameter.
														Currently defined flags are:
															PARAMETER_WAS_PROMPTED_FOR - the
																parameter was prompted for.
																In other words a valid parameter
																was not on the command line and
																the parameter needed to be
																prompted for.
															PARAMETER_DEFAULT_WAS_USED -
																prompting did not take place
																since it was determined that
																a new frame was asked for and
																the default value that was
																passed in was returned.

  Returns:		ODISTAT

  Abstract:		GetParameter is called to search the search string for the
					keyword if not found and the word is required CMSMPromptUserForInput
					will be called to try and obtain the value.  After the value
					is found this routine validates that it is of the correct type.

  Notes:				

=======================================================================*/
ODISTAT	CMSMGetParameter(
					MEON_STRING			*Keyword,
					SCREEN_HANDLE     *screenID,
					MEON_STRING			*commandLine,
               BOOLEAN				needToPrompt,
					MEON_STRING			*prompt,
					MEON_STRING			*Default,
					UINT16				Flags,
					UINT16				ConversionSpecifierCount,
					MEON_STRING			*param,
					UINT32				*paramFlags)
{
	ODISTAT	status;
	MEON_STRING		*s = NULL;
	MEON_STRING		*separators = CMSM_PARSER_WHITESPACE_EQUAL_SEPARATORS_MSG;
	MEON_STRING		*whiteSpace = CMSM_PARSER_WHITESPACE_SEPARATORS_MSG;
	MEON_STRING		*validInts = CMSM_PARSER_VALID_INTS_MSG;
	MEON_STRING		*validUInts = CMSM_PARSER_VALID_U_INTS_MSG;
	MEON_STRING		*validXInts	= CMSM_PARSER_VALID_X_INTS_MSG;
	MEON_STRING		*validStrings = CMSM_PARSER_VALID_STRINGS_MSG;
	MEON_STRING		*validCharSet;
	int		parmCount;
	UINT32	len;
#ifndef OS_NT
	MEON_STRING		yesNoBuff[3];
#endif

	*paramFlags = DEFAULT_PARAMETER_FLAGS;

	/* Lets setup the allowed character set in case we have to prompt the user. */
	switch (Flags & TYPEMASK)
	{
		case STRINGTYPE:
			validCharSet = validStrings;
			break;
		case INTTYPE:
			validCharSet = validInts;
			break;
		case UINTTYPE:
			validCharSet = validUInts;
			break;
		case XINTTYPE:
			validCharSet = validXInts;
			break;
	}


	/* We need to UPCASE the Keyword. */
   CMSMstrupr(Keyword);

	/* Find the keyword in the command line. */
#ifdef OS_NT
	if (commandLine != NULL)
#else
	if (commandLine != NULL && !ForcePrompt)
#endif
	{
		s = commandLine;

		while (s != NULL)
		{
			s = CMSMstrstr(s, Keyword);
			if (s == NULL)
				break;
			/* Now lets make sure this is the keyword and not part of a */
			/* different word. */
			len = CMSMstrlen(Keyword);
			if ((CMSMstrcspn(&s[len], separators) == 0) || s[len] == '\0')
			{
            /* If s is at the beginning of the commandLine or there is a */
				/* separator preceeding s, then we matched. */
            if (s == commandLine || (CMSMstrchr(separators, *(s-1)) != NULL))
				{
					break;
				}
			}
			s += len;
		}
	}

#ifdef OS_NT
	if (s == NULL)
	{
		/* The keyword was not on the command line check if required. */
		if (Flags & REQUIREDPARAM)
		{
			return(ODISTAT_FAIL);
		}
		else
		{
			return ODISTAT_ITEM_NOT_PRESENT;
		}
	}
#else
	if (s == NULL || ForcePrompt)
	{
		/* The keyword was not on the command line check if required. */
		if (Flags & REQUIREDPARAM)
		{
         /* If we need to prompt for frame doit. */
         if ( (AskIfNewFrame) &&
				  !(CurrentConfigTable->MLIDCFG_FrameID & ASK_FRAME_DONE) )
         {
   		   status = CMSMPromptUserForInput(
                        screenID,
   	      				CMSM_PARSER_VALID_YES_NO_MSG,
	   	      			CMSM_PARSER_LOAD_ANOTHER_FRAME_MSG,
		   	      		CMSM_PARSER_N_MSG,
			   	      	yesNoBuff,
								2);

				if (status == ODISTAT_FAIL)
					return(ODISTAT_FAIL);

				CurrentConfigTable->MLIDCFG_FrameID |= ASK_FRAME_DONE;

      		if (yesNoBuff[0] == 'y' || yesNoBuff[0] == 'Y')
	      	{
               NewFrame = TRUE;
				}
            AskIfNewFrame = FALSE;
         }

		   if ( (NewFrame == TRUE) &&
				 !(CurrentConfigTable->MLIDCFG_FrameID & CHECK_MULTICARD_DONE) &&
				 !(TSMParseFlag) )
         {
				/* Set param to Default so that we have a valid response. */
				*paramFlags |= PARAMETER_DEFAULT_WAS_USED;
				CMSMstrcpy(param, Default);
				return(ODISTAT_SUCCESSFUL);
         }

         if (needToPrompt == TRUE)
         {
     			status = CMSMPromptUserForInput(
      						screenID,
	      					validCharSet,
		      				prompt,
			      			Default,
				      		param,
					      	MAX_PARAM_LEN);

     			if (status == ODISTAT_FAIL)
				{
      			return(ODISTAT_FAIL);
				}
				*paramFlags |= PARAMETER_WAS_PROMPTED_FOR;
         }
         else
			{
				return ODISTAT_SUCCESSFUL;
			}
		}
		else
		{
         /* The parameter was not on the command line.  Check to see if we */
         /* were asked to prompt due to a previous problem. */
			if (ForcePrompt)
			{
				status = CMSMPromptUserForInput(
							screenID,
							validCharSet,
							prompt,
							Default,
							param,
							MAX_PARAM_LEN);
				if (status == ODISTAT_FAIL)
				{
					return ODISTAT_FAIL;
				}
				*paramFlags |= PARAMETER_WAS_PROMPTED_FOR;
			}
			else
			{
				return ODISTAT_ITEM_NOT_PRESENT;
			}
		}
	}
#endif
	else
	{
		/* The keyword has been found or we will not get to here. */
		/* Copy the parameter to the return string. */
		s += CMSMstrlen(Keyword);
		s += CMSMstrspn(s, separators);
		CMSMstrncpy(param, s, MAX_PARAM_LEN);
	}

#ifndef OS_NT
CheckValidityAgain:
#endif

	/* Now lets check the input for vlidity. */
	s = param;

	/* lets start at the first entered field. */
	s += CMSMstrspn(s, whiteSpace);

	if (*s == '\0')
	{
		/* A parameter was not supplied with the keyword. */
		status = ODISTAT_BAD_PARAMETER;

#ifdef OS_NT
		CMSMLogEvent(
			MSG_TYPE_INIT_WARNING,
			KEYWORD_VALUE_MISSING,
			Keyword);
#else
		CMSMPrintString(NULL,
			MSG_TYPE_INIT_WARNING,
			CMSM_PARSER_KEYWORD_VALUE_MISSING_MSG,
			Keyword,
			NULL);
#endif
	}
	else
	{
		/* Assume that we will be successful. */
		status = ODISTAT_SUCCESSFUL;

		for (parmCount = 0;
			parmCount < ConversionSpecifierCount && status == ODISTAT_SUCCESSFUL;
				++parmCount)
		{
			/* lets check to see if the type is correct. */
			/* if *s is not NULL lets check to see if type of field is correct. */
			if (*s)
			{
				switch (Flags & TYPEMASK)
				{
					case STRINGTYPE:
						if ((s += CMSMstrcspn(s, whiteSpace)) == 0)
						{
							/* The field type is invalid. */
							status = ODISTAT_BAD_PARAMETER;
						}
						break;
					case INTTYPE:
					case UINTTYPE:
					case XINTTYPE:
						if ((s += CMSMstrspn(s, validCharSet)) == 0)
						{
							/* The field type is invalid. */
							status = ODISTAT_BAD_PARAMETER;
						}
						break;
				}
			}

			/* Eat the wite space. */
			s += CMSMstrspn(s, whiteSpace);
		}
	}

	if (status != ODISTAT_SUCCESSFUL)
	{
#ifdef OS_NT
		/* The input was invalid. */
		if (Flags & REQUIREDPARAM)
		{
         return ODISTAT_FAIL;
		}
#else
		/* The input was invalid lets try again. */
		if ((Flags & REQUIREDPARAM) || ForcePrompt)
		{
			/* The keyword is required prompt user for the keyword. */
			status = CMSMPromptUserForInput(
							screenID,
							validCharSet,
							prompt,
							Default,
							param,
							MAX_PARAM_LEN);

			if (status == ODISTAT_FAIL)
			{
				return(ODISTAT_FAIL);
			}

			*paramFlags |= PARAMETER_WAS_PROMPTED_FOR;
		}
#endif
		else
		{
			return ODISTAT_ITEM_NOT_PRESENT;
		}

#ifndef OS_NT
		goto CheckValidityAgain;
#endif
	}

	if ((Flags & TYPEMASK) == STRINGTYPE)
	{
		param[s - param] = '\0';
	}
	else
	{
		/* This is a numeric type or a enumed string.  If an enumed string and */
		/* the parameter is on the command line, then param will hold the */
		/* string of the parameter.  If the parameter was promped for then */
		/* the index number will be in param.  In either case we need to null */
		/* terminate the parameter string espically if the parameter was on */
		/* the command line so that we do not get the parameter and the rest */
		/* of the command line in param. */
		len = s - param;
		len += CMSMstrcspn(s, whiteSpace);
		param[len] = '\0';
	}

	return(ODISTAT_SUCCESSFUL);					
}



/*======================================================================

  Procedure Name:	CMSMFindParameter

  Arguments:	
					Keyword							- Ptr to the keyword to be found
					screenID          			- Screen Handle for user I/O
					commandLine						-	Ptr to the String to search.
					defaultValue							- Ptr to a default value of the type spec'ed by Flags
					ConversionSpecifier			- Ptr to a scanf type conversion specifier
					Flags								- 0x0001 - set if the parameter is required
															- 0x0002 - set if there is a default
															- 0x0004 - set if the parameter is a string
															- 0x0008 - set if the parameter is an int
															- 0x0010 - set if the parameter is an unsigned int
															- 0x0020 - set if the parameter is an unsigned hexidecimal int
					ParameterType					- 0x0000 if custom, otherwise one bit set for which
															standard param is to be found.
					ConversionSpecifierCount	- count of the number of %s, %c, %[] spec's are
															in ConversionSpecifier.
					param								- Output param, to be set to the paramter value found.

  Returns:		ODISTAT

  Abstract:		CMSMFindParameter is used to obtain the "value" of an unconstrained
					string or numeric parameter. CMSMFindParameter takes a pointer to
					the keyword to be found, a pointer to a default value (the
					pointer my be NULL), a pointer to a scanf style conversion
					specifier string , two flags parameters, and a count of the
					number of conversion specifiers in conversion specifier string.
					If the flags 0x0001 bit is set if the parameter is required,
					the 0x0002 bit is set if there is a default, the 0x0004 bit is
					set if the parameter is a string, the 0x0008 bit is set if the
					parameter is an int, the 0x0010 bit is set if the parameter is
					an unsigned int, and the 0x0020 bit is set if the parameter is
					an unsigned hexidecimal int.  The type of the default value and
					the conversion specifier will match the type specified by the
					Flags parameter.  ParameterType is passed to allow
					CMSMFindParameter to know whether to use the keyword passed or use
					its own (platform dependant) keyword.  If the parameter is
					custom, no bits will be set in ParamterType, otherwise one bit
					will be set specifying which standard parameter is being
					parsed.

					If CMSMFindParameter finds the param, it returns ODISTAT_SUCCESSFUL
					and a pointer to the first char of the parameter's value as a
					string (the pointer must NOT point to any separator charater(s)
					between the keyword and the value.)  The parameter must be
					terminated by some form of whitespace acceptable to ANSI C
					sscanf.  Also, the entire parameter must be present; that is,
					if ConversionSpecifier consists of multiple %s's, %c's, and/or
					%[]'s and the user parameter contains embedded whitespace, the
					value returned by FindParamter must contain at least MIN (the
					number of %s's, the number of whitespace separated fields the
					user entered) whitespace separated fields.  Futhermore, if the
					user entered fewer fields than there are %s's the returned
					parameter must end with enough NULLs to satisfy the remaining
					%s's.

					If the parameter is optional and not found, the function should
					return ODISTAT_ITEM_NOT_PRESENT.  If the parameter is required,
					not present, and can not be obtained, the function should
					return ODISTAT_FAIL. This will cause the parsing to fail and
					the driver will not load.

  Notes:			The keyword passed is generic and CMSMFindParameter must recognize
					synonyms and platform specific alternate spellings.	

=======================================================================*/
ODISTAT CMSMFindParameter(
					MEON_STRING			*Keyword,
					SCREEN_HANDLE     *screenID,
					MEON_STRING			*commandLine,
					void					*defaultValue,
					MEON_STRING			*ConversionSpecifier,
					UINT16				Flags,
					UINT16				ParameterType,
					UINT16				ConversionSpecifierCount,
					MEON_STRING			*param)
{
#ifndef OS_NT
   MEON_STRING		defaultString[MAX_PARAM_LEN];
	MEON_STRING  	*promptString;
#endif
   ODISTAT  status;
	UINT32	paramFlags;


   /* Variables that are not used are set to themselves. */
   /* this will get rid of the compiler warning. */
   ConversionSpecifier = ConversionSpecifier;
   ParameterType = ParameterType;

#ifndef OS_NT
   /* We need to build the prompt string. */
	promptString = CMSMInitAlloc(MAX_PROMPT_SIZE);

	defaultString[0] = '\0';
	if (promptString)
	{
		CMSMstrcpy(promptString, CMSM_PARSER_ENTER1_VALUE_MSG);
		CMSMstrcat(promptString, Keyword);
		CMSMstrcat(promptString, CMSM_PARSER_SPACE_COLON_SPACE_MSG);

		switch (Flags & TYPEMASK)
		{
			case INTTYPE:
			case UINTTYPE:
				if (Flags & DEFAULTPRESENT)
				{
					CMSMitoa(defaultString, *(int *)defaultValue);
				}
				break;

			case XINTTYPE:
				if (Flags & DEFAULTPRESENT)
				{
					CMSMxtoa(defaultString, *(int *)defaultValue);
				}
				break;

			default: /* STRINGTYPE: */
				if (Flags & DEFAULTPRESENT)
				{
					CMSMstrncpy(defaultString, defaultValue, MAX_PARAM_LEN);
				}
				break;
		}
	}
#endif

	/* Lets go get the value. */
#ifdef OS_NT
	status = CMSMGetParameter(
					Keyword,
               screenID,
					commandLine,
               TRUE,
					NULL,
					defaultValue,
					Flags,
					ConversionSpecifierCount,
					param,
					&paramFlags);
#else
	status = CMSMGetParameter(
               Keyword,
               screenID,
               commandLine,
               TRUE,
               promptString,
               defaultString,
               Flags,
               ConversionSpecifierCount,
               param,
					&paramFlags);

	/* Now that we are done with the prompt FREE IT! */
	/* The dirverData parameter in the CMSMFree call is NULL in this case */
	/* since the MSM is doing the alloc and free of the memory. */

	if (promptString)
		CMSMFree(NULL, promptString);
#endif


   return status;
}



/*======================================================================
  Procedure Name:	CMSMFindEnumParameter

  Arguments:	
					Keyword							- Ptr to the keyword to be found
					screenID          			- Screen Handle for user I/O
					commandLine						-	Ptr to the String to search.
					defaultValue							- Ptr to a default value of the type spec'ed by Flags
					ConversionSpecifier			- Ptr to a scanf type conversion specifier
					Flags								- 0x0001 - set if the parameter is required
															- 0x0002 - set if there is a default
															- 0x0004 - set if the parameter is a string
															- 0x0008 - set if the parameter is an int
															- 0x0010 - set if the parameter is an unsigned int
															- 0x0020 - set if the parameter is an unsigned hexidecimal int
					ParameterType					- 0x0000 if custom, otherwise one bit set for which
															standard param is to be found.
					OptionPtr						- Ptr to a structure containing a list of the
															valid values for this parameter.  These values
															are of type specified by Flags.
					param								- Output param, to be set to the paramter value found.

  Returns:		ODISTAT

  Abstract:		CMSMFindEnumParameter is used to obtain the "value" of an
					enumeration	parameter.

					CMSMFindEnumParameter is the same as CMSMFindParameter except
					that no ConversionSpecifierCount is passed and a pointer to an
					OptionStruct is passed.  The values	in this structure will be
					the same type as specified by the Flags parameter.
  Notes:				

=======================================================================*/
ODISTAT CMSMFindEnumParameter(
					MEON_STRING			*Keyword,
					SCREEN_HANDLE     *screenID,
					MEON_STRING			*commandLine,
					void              *defaultValue,
					MEON_STRING       *ConversionSpecifier,
					UINT16            Flags,
					UINT16				ParameterType,
					DRIVER_OPTION 		*OptionPtr,
					MEON_STRING       *param)
{
	ODISTAT					status = ODISTAT_ITEM_NOT_PRESENT;
   PARAMETER_OPTIONS	   *options;
	int						ivalue;
	UINT32					i;
	UINT32					uivalue;
	UINT32					paramFlags;
#ifdef OS_NT
	BOOLEAN					needToPrompt = FALSE;
#else
	BOOLEAN					found = FALSE;
	BOOLEAN					needToPrompt = TRUE;
	ODISTAT					conflict;
	MEON_STRING				*promptString;
	MEON_STRING				tempString[MAX_PARAM_LEN];
   MEON_STRING				defaultString[MAX_PARAM_LEN];
	int						j;
	UINT16					newFlags;
#endif

   defaultValue = defaultValue;

   /* Variables that are not used are set to themselves. */
   /* this will get rid of the compiler warning. */
   ConversionSpecifier = ConversionSpecifier;

#ifdef OS_NT
	options = OptionPtr->Parameter0.OptionPtr;
#else
	/* newFlags is used so that we can prompt for an index instead of */
	/* the string for String types. */
	newFlags = Flags;

   /* We need to build the prompt string. */
	promptString = CMSMInitAlloc(MAX_PROMPT_SIZE);

	defaultString[0] = '\0';
	if (promptString)
	{
		CMSMstrcpy(promptString, CMSM_PARSER_SUPPORTED_MSG);
		CMSMstrcat(promptString, Keyword);
		CMSMstrcat(promptString, CMSM_PARSER_VALUES_ARE_MSG);

		/* Add the list of values to the prompt string. */
		switch (Flags & TYPEMASK)
		{
			case STRINGTYPE:
				newFlags &= ~STRINGTYPE;
				newFlags |= INTTYPE;

            /* Lets use 1 as the default. */
            CMSMstrcpy(defaultString, CMSM_PARSER_1_MSG);

			/* make our own copy of hsms option list */
				options = CMSMInitAlloc((OptionPtr->Parameter0.OptionPtr->OptionCount
												* sizeof(MEON_STRING *)) + sizeof(UINT32));
				if (!options)
				{
					CMSMFree(NULL, promptString);		
               return ODISTAT_FAIL;
				}
			
				options->OptionCount = OptionPtr->Parameter0.OptionPtr->OptionCount;

				for (i = 0; i < options->OptionCount; ++i)
					(options->ENUM_LIST.StrOptVal)[i] =
					(OptionPtr->Parameter0.OptionPtr->ENUM_LIST.StrOptVal)[i];

            if (options->OptionCount <= 1)
            {
               /* We don't need to prompt use the default. */
					CMSMstrcpy(param, (options->ENUM_LIST.StrOptVal)[0]);
  					OptionPtr->Parameter1.Max = 0;
               needToPrompt = FALSE;
  					break;
            }
				for (i = 0; i < options->OptionCount; ++i)
				{
					/* This is a string no conversion needed. */
					CMSMstrcat(promptString, CMSM_PARSER_NEWLINE_MSG);
					CMSMitoa(tempString, i + 1);
					CMSMstrcat(promptString, tempString);
					CMSMstrcat(promptString, CMSM_PARSER_PERIOD_SPACE_MSG);
					CMSMstrcat(promptString, (options->ENUM_LIST.StrOptVal)[i]);
				}

				break;
			case INTTYPE:
			case UINTTYPE:

				/* make our own copy of hsms option list */
				options = CMSMInitAlloc((OptionPtr->Parameter0.OptionPtr->OptionCount+1)
												 * sizeof(UINT32));
				if (!options)
				{
					CMSMFree(NULL, promptString);		
               return ODISTAT_FAIL;
				}
			
				options->OptionCount = OptionPtr->Parameter0.OptionPtr->OptionCount;

				for (i = 0; i < options->OptionCount; ++i)
					options->ENUM_LIST.NumOptVal[i] =
					OptionPtr->Parameter0.OptionPtr->ENUM_LIST.NumOptVal[i];

            /* Assume first will be the Default. */
            CMSMitoa(defaultString, options->ENUM_LIST.NumOptVal[0]);
				OptionPtr->Parameter1.Max = options->ENUM_LIST.NumOptVal[0];

            /* Lets check to see if we have any options to choose from. */
				for (i = 0, j = 0; i < options->OptionCount; ++i)
				{
               /* We need to check for hardware conflicts. */
               conflict = CMSMCheckForHardwareConflict(
                        options->ENUM_LIST.NumOptVal[i],
              				ParameterType);

               if (conflict == ODISTAT_SUCCESSFUL)
               {
   					/* This is of type integer we need to convert to string. */
	   				CMSMitoa(tempString, options->ENUM_LIST.NumOptVal[i]);
		   			CMSMstrcat(promptString, tempString);
			   		CMSMstrcat(promptString, CMSM_PARSER_COMMA_SPACE_MSG);
                  options->ENUM_LIST.NumOptVal[j] = options->ENUM_LIST.NumOptVal[i];
                  ++j;
               }
				}
            options->OptionCount = j;

            if (options->OptionCount > 1)
            {
               /* We have some valid Choices to choose from. */
               /* USE the first in the list for the default. */
               CMSMitoa(defaultString, options->ENUM_LIST.NumOptVal[0]);
            }
            else
            {
               /* We don't need to prompt use the default. */
               needToPrompt = FALSE;

               /* If we have one option use it. */
               if (options->OptionCount)
                  CMSMitoa(param, options->ENUM_LIST.NumOptVal[0]);
               else
   					CMSMstrcpy(param, defaultString);
            }


				/* Now lets remove the last ", " that was added. */
				promptString[CMSMstrlen(promptString) -2] = '\0';

				break;
			case XINTTYPE:

				/* make our own copy of hsms option list */
				options = CMSMInitAlloc((OptionPtr->Parameter0.OptionPtr->OptionCount+1)
												* sizeof(UINT32));
				if (!options)
				{
					CMSMFree(NULL, promptString);		
               return ODISTAT_FAIL;
				}
			
				options->OptionCount = OptionPtr->Parameter0.OptionPtr->OptionCount;

				for (i = 0; i < options->OptionCount; ++i)
					options->ENUM_LIST.NumOptVal[i] =
					OptionPtr->Parameter0.OptionPtr->ENUM_LIST.NumOptVal[i];

            /* Assume first will be the Default. */
            CMSMxtoa(defaultString, options->ENUM_LIST.NumOptVal[0]);
				OptionPtr->Parameter1.Max = options->ENUM_LIST.NumOptVal[0];

            /* Lets check to see if we have any options to choose from. */
				for (i = 0, j= 0; i < options->OptionCount; ++i)
				{
               /* We need to check for hardware conflicts. */
               conflict = CMSMCheckForHardwareConflict(
                        options->ENUM_LIST.NumOptVal[i],
              				ParameterType);

               if (conflict == ODISTAT_SUCCESSFUL)
               {
   					/* This is of type xinteger we need to convert to string. */
	   				CMSMxtoa(tempString, options->ENUM_LIST.NumOptVal[i]);
		   			CMSMstrcat(promptString, tempString);
			   		CMSMstrcat(promptString, CMSM_PARSER_COMMA_SPACE_MSG);
                  options->ENUM_LIST.NumOptVal[j] = options->ENUM_LIST.NumOptVal[i];
                  ++j;
               }
				}
            options->OptionCount = j;

            if (options->OptionCount > 1)
            {
               /* We have some valid Choices to choose from. */
               /* USE the first in the list for the default. */
               CMSMxtoa(defaultString, options->ENUM_LIST.NumOptVal[0]);
            }
            else
            {
               /* We don't need to prompt use the default. */
               needToPrompt = FALSE;

               /* If we have one option use it. */
               if (options->OptionCount)
                  CMSMxtoa(param, options->ENUM_LIST.NumOptVal[0]);
               else
   					CMSMstrcpy(param, defaultString);
            }


				/* Now lets remove the last ", " that was added. */
				promptString[CMSMstrlen(promptString) -2] = '\0';

				break;

		}
		/* Now add the keyword and : to the prompt. */
		CMSMstrcat(promptString, CMSM_PARSER_NEWLINE_MSG);
		CMSMstrcat(promptString, Keyword);
		CMSMstrcat(promptString, CMSM_PARSER_SPACE_COLON_SPACE_MSG);

	}
#endif

	/* We need to special case the following standard keywords. */
	/* INTPARAM, PORTPARAM, DMAPARAM, MEMPARAM. */
	switch (ParameterType)
	{
		case INTPARAM:
			status = CMSMGetParameter(
                     CMSM_PARSER_INTO_TXTMSG,
                     screenID,
							commandLine,
                     needToPrompt,
#ifdef OS_NT
                     NULL,
							NULL,
							(UINT16)(Flags & ~REQUIREDPARAM),
#else
                     promptString,
							defaultString,
							newFlags & ~REQUIREDPARAM,
#endif
							1,
							param,
							&paramFlags);

			break;
		case PORTPARAM:
			status = CMSMGetParameter(
							CMSM_PARSER_PORT0_TXTMSG,
                     screenID,
							commandLine,
                     needToPrompt,
#ifdef OS_NT
                     NULL,
							NULL,
							(UINT16)(Flags & ~REQUIREDPARAM),
#else
                     promptString,
							defaultString,
							newFlags & ~REQUIREDPARAM,
#endif
							1,
							param,
							&paramFlags);

			break;
		case DMAPARAM:
			status = CMSMGetParameter(
							CMSM_PARSER_DMA0_TXTMSG,
                     screenID,
							commandLine,
                     needToPrompt,
#ifdef OS_NT
                     NULL,
							NULL,
							(UINT16)(Flags & ~REQUIREDPARAM),
#else
                     promptString,
							defaultString,
							newFlags & ~REQUIREDPARAM,
#endif
							1,
							param,
							&paramFlags);

			break;
		case MEMPARAM:
			status = CMSMGetParameter(
							CMSM_PARSER_MEM0_TXTMSG,
                     screenID,
							commandLine,
                     needToPrompt,
#ifdef OS_NT
                     NULL,
							NULL,
							(UINT16)(Flags & ~REQUIREDPARAM),
#else
                     promptString,
							defaultString,
							newFlags & ~REQUIREDPARAM,
#endif
							1,
							param,
							&paramFlags);

			break;
#ifndef OS_NT
		case FRAMEPARAM:
			if (options->OptionCount <= 1)
				ForcePrompt = TRUE;	 	/* Need to do this here
										   else in certain cases we get bogus
										   FRAME prompts	*/
			break;
#endif
	}

#ifdef OS_NT
	if (status == ODISTAT_ITEM_NOT_PRESENT)
   {
		status = CMSMGetParameter(
						Keyword,
                  screenID,
						commandLine,
                  needToPrompt,
                  NULL,
						NULL,
						Flags,
						1,
						param,
						&paramFlags);
	}

  	/* Now lets check to see if the entered value matches one of ours. */
  	if (status == ODISTAT_SUCCESSFUL)
   {
	   switch (Flags & TYPEMASK)
  		{
  			case STRINGTYPE:
            /* If the keyword was on the Command line we need check it */
            /* against the words in the list. */
            s = param;
				/* If we get here the keyword was on the command line check */
				/* for validity. */
				for (i = 0; i < options->OptionCount; ++i)
				{
					if (CMSMstrcmp(param, options->ENUM_LIST.StrOptVal[i]) == 0)
						break;
				}
            if (i < options->OptionCount)
   				OptionPtr->Parameter1.Max = i;
				else return ODISTAT_FAIL;
            break;
			case INTTYPE:
	  			/* We need to se if this matches one of our values. */
	   		ivalue = CMSMatoi(param);

  		   	for (i = 0; i < options->OptionCount; ++i)
   		   {
   			   if (ivalue == options->ENUM_LIST.NumOptVal[i])
	  				   break;
	     		}
  	      	if (i >= options->OptionCount)
					return ODISTAT_FAIL;
      		break;
		   case UINTTYPE:
	  			/* We need to se if this matches one of our values. */
			   uivalue = CMSMatoi(param);

   			for (i = 0; i < options->OptionCount; ++i)
				{
	  				if (uivalue == options->ENUM_LIST.UNumOptVal[i])
	    				break;
	      	}
		      if (i >= options->OptionCount)
					return ODISTAT_FAIL;
   			break;
			case XINTTYPE:
	  			/* We need to se if this matches one of our values. */
	  			uivalue = CMSMatox(param);

      		for (i = 0; i < options->OptionCount; ++i)
		   	{
	  		   	if (uivalue == options->ENUM_LIST.UNumOptVal[i])
				   	break;
      		}
   			if (i >= options->OptionCount)
					return ODISTAT_FAIL;
	   		break;
   	}
	}
#else
	if (status == ODISTAT_ITEM_NOT_PRESENT)
   {
		while (!found)
		{
			status = CMSMGetParameter(
							Keyword,
                     screenID,
							commandLine,
                     needToPrompt,
                     promptString,
							defaultString,
							newFlags,
							1,
							param,
							&paramFlags);

			ForcePrompt = FALSE;

   		/* Now lets check to see if the entered value matches one of ours. */
	   	if (status == ODISTAT_SUCCESSFUL)
		   {
			   switch (Flags & TYPEMASK)
   			{
					case STRINGTYPE:
						/* Check to see if the parameter was prompted for.  If it */
						/* was prompted for then the parameter is an index into the */
						/* option list.  If the default was used, the index is */
						/* as if the parameter was prompted for and the default */
						/* index was returned.  If the parameter was on the command */
						/* line, it is the parameter itself. */
						if (paramFlags &
							(PARAMETER_WAS_PROMPTED_FOR | PARAMETER_DEFAULT_WAS_USED))
						{
							/* This is an enumerated string so we need to use the */
							/* index returned. */
							ivalue = CMSMatoi(param);
							if (--ivalue < options->OptionCount)
							{
								CMSMstrcpy(param,
									(options->ENUM_LIST.StrOptVal)[ivalue]);
								OptionPtr->Parameter1.Max = ivalue;
								found = TRUE;
							}
							else
							{
								/* The index returned is out of range.  Setup */
								/* to go through the loop again and repormpt. */
								ForcePrompt = TRUE;
								needToPrompt = TRUE;
							}
						}
						else
						{
							/* If we get here the keyword was on the command line */
							/* check for validity. */
							for (i = 0; i < options->OptionCount; ++i)
							{
								/* Since the command line has been upper cased, we */
								/* need to upper case the option string so that */
								/* the string compare will succeed. */
								CMSMstrcpy(tempString,
									(options->ENUM_LIST.StrOptVal)[i]);
								CMSMstrupr(tempString);
								if (CMSMstrcmp(param, tempString) == 0)
								{
									/* Copy the case of the string into param so that */
									/* it will be the case that the driver requested. */
									CMSMstrcpy(param,
										(options->ENUM_LIST.StrOptVal)[i]);
									break;
								}
							}
							if (i < options->OptionCount)
							{
								OptionPtr->Parameter1.Max = i;
								found = TRUE;
							}
							else
							{
								/* The parameter on the command was incorrect. */
								/* Go through the loop again and force a prompt. */
								ForcePrompt = TRUE;
								needToPrompt = TRUE;
							}
						}
                  break;
	   			case INTTYPE:
		   			/* We need to se if this matches one of our values. */
			   		ivalue = CMSMatoi(param);

                  if (options->OptionCount)
                  {
     				   	for (i = 0; i < options->OptionCount; ++i)
      				   {
	      				   if (ivalue == options->ENUM_LIST.NumOptVal[i])
		      				   break;
  			      		}
   			      	if (i < options->OptionCount)
					      {
	      			      found = TRUE;
   			   		}
   	   				else
							{
					/* check to see if value matches previosly loaded boards frame	*/
								if ((MatchBoardNum =
									CMSMCheckDuplicateEntry(&ivalue, ParameterType)) != -1)
								{	/* TNL The options matches that of previosly loaded brd */
								 	found = TRUE;
									status = ODISTAT_DUPLICATE_ENTRY;	/* Flag it	*/
								}
								else
								{
		   	   			/* The returned value was wrong. */
			   	   		/* Set the command line to NULL incase it was bad. */
									ForcePrompt = TRUE;
									needToPrompt = TRUE;
								}
  					   	}
                  }
                  else
                  {
                     /* This is for a new frame. */
      			      found = TRUE;
                  }
   		   		break;
				   case UINTTYPE:
		   			/* We need to se if this matches one of our values. */
					   uivalue = CMSMatoi(param);

                  if (options->OptionCount)
                  {
      					for (i = 0; i < options->OptionCount; ++i)
	      				{
		      				if (uivalue == options->ENUM_LIST.UNumOptVal[i])
			      				break;
				      	}
					      if (i < options->OptionCount)
						   {
						      found = TRUE;
                     }
   						else
	   					{
							/* check to see if value matches previosly loaded boards frame	*/
								if ((MatchBoardNum =
									CMSMCheckDuplicateEntry(&uivalue, ParameterType)) != -1)
								{	/* The options matches that of previosly loaded brd */
								 	found = TRUE;
									status = ODISTAT_DUPLICATE_ENTRY;	/* Flag it	*/
								}
								else
								{

		   					/* The returned value was wrong. */
			   				/* Set the command line to NULL incase it was bad. */
									ForcePrompt = TRUE;
									needToPrompt = TRUE;
								}
					   	}
                  }
                  else
                  {
                     /* This is for a new frame. */
                     found = TRUE;
                  }
   					break;
	   			case XINTTYPE:
		   			/* We need to se if this matches one of our values. */
		   			uivalue = CMSMatox(param);

                  if (options->OptionCount)
                  {
   			   		for (i = 0; i < options->OptionCount; ++i)
	   			   	{
			   		   	if (uivalue == options->ENUM_LIST.UNumOptVal[i])
		   				   	break;
   			   		}
   	   				if (i < options->OptionCount)
	   					{
		   				   found = TRUE;
                     }
		   				else
			   			{
							/* check to see if value matches previosly loaded boards frame	*/
								if ((MatchBoardNum =
									CMSMCheckDuplicateEntry(&uivalue, ParameterType)) != -1)
								{	/* TNL The option matches that of previosly loaded brd */
								 	found = TRUE;
									status = ODISTAT_DUPLICATE_ENTRY;	/* Flag it	*/
								}
								else
								{

				   			/* The returned value was wrong. */
					   		/* Set the command line to NULL incase it was bad. */
									ForcePrompt = TRUE;
									needToPrompt = TRUE;		/* TNL 9/15/95	*/
								}
   						}
                  }
                  else
                  {
                     /* This is for a new frame. */
                     found = TRUE;
                  }
			   		break;
   			}
	   	}
		   else
   		{
				/* The dirverData parameter in the CMSMFree call is NULL in this */
				/* case since the MSM is doing the alloc and free of the memory. */
	   		CMSMFree(NULL, promptString);
				CMSMFree(NULL, options);
		   	return(status);
   		}
      }
	}

	/* The dirverData parameter in the CMSMFree call is NULL in this case */
	/* since the MSM is doing the alloc and free of the memory. */
	CMSMFree(NULL, promptString);
	CMSMFree(NULL, options);
#endif

	return(status);
}



/*======================================================================
  Procedure Name:	CMSMFindRangeParameter

  Arguments:	
					Keyword                    - Ptr to the keyword to be found
					screenID          			- Screen Handle for user I/O
					commandLine						-	Ptr to the String to search.
					defaultValue                    - Ptr to a default value of the type spec'ed by Flags
					ConversionSpecifier			- Ptr to a scanf type conversion specifier
					Flags									- 0x0001 - set if the parameter is required
															- 0x0002 - set if there is a default
															- 0x0004 - set if the parameter is a string
															- 0x0008 - set if the parameter is an int
															- 0x0010 - set if the parameter is an unsigned int
															- 0x0020 - set if the parameter is an unsigned hexidecimal int
					ParameterType					- 0x0000 if custom, otherwise one bit set for which
															standard param is to be found.
					Min								- The minimum value allowed.
					Max								- The maximum value allowed.
					param								- Output param, to be set to the paramter value found.

  Returns:		ODISTAT

  Abstract:		CMSMFindRangeParameter is used to obtain the "value" of a numeric
					range parameter.

					CMSMFindRangeParameter is the same as CMSMFindParameter except
					that no ConversionSpecifierCount is passed and two parameters
					specifying the minimum and maximum are passed.  The Flags
					parameter will specify one of the	numeric types.

  Notes:				

=======================================================================*/
ODISTAT CMSMFindRangeParameter(
					MEON_STRING			*Keyword,
					SCREEN_HANDLE     *screenID,
					MEON_STRING			*commandLine,
					void					*defaultValue,
					MEON_STRING			*ConversionSpecifier,
					UINT16				Flags,
					UINT16				ParameterType,
					int					Min,
					int					Max,
					MEON_STRING			*param)
{
	ODISTAT					status = ODISTAT_ITEM_NOT_PRESENT;
	int						ivalue;
	UINT32					uivalue;
	UINT32					paramFlags;
#ifndef OS_NT
	MEON_STRING				*promptString, tempString[20];
   MEON_STRING          defaultString[MAX_PARAM_LEN];
	BOOLEAN					found = FALSE;
#endif

   /* Variables that are not used are set to themselves. */
   /* this will get rid of the compiler warning. */
   ConversionSpecifier = ConversionSpecifier;

#ifndef OS_NT
   /* We need to build the prompt string. */
	promptString = CMSMInitAlloc(MAX_PROMPT_SIZE);

	defaultString[0] = '\0';
	if (promptString)
	{
		CMSMstrcpy(promptString, CMSM_PARSER_ENTER_VALUE_MSG);
		CMSMstrcat(promptString, Keyword);
		CMSMstrcat(promptString, CMSM_PARSER_BETWEEN_MSG);

		/* Add the range of values to the prompt string. */
		switch (Flags & TYPEMASK)
		{
			case STRINGTYPE:
				status = ODISTAT_FAIL;

            /* Copy the Default value to defaultString. */
            if (Flags & DEFAULTPRESENT)
               CMSMstrncpy(defaultString, defaultValue, MAX_PARAM_LEN);

				break;
			case INTTYPE:
			case UINTTYPE:
				/* This in of type integer we need to convert to string. */
				CMSMitoa(tempString, Min);
				CMSMstrcat(promptString, tempString);
				CMSMstrcat(promptString, CMSM_PARSER_SPACE_MINUS_SPACE_MSG);
				CMSMitoa(tempString, Max);
				CMSMstrcat(promptString, tempString);

            /* Copy the Default value to defaultString. */
            if (Flags & DEFAULTPRESENT)
               CMSMitoa(defaultString, *(int *)defaultValue);

				break;
			case XINTTYPE:
			   /* This in of type integer we need to convert to string. */
				CMSMxtoa(tempString, Min);
				CMSMstrcat(promptString, tempString);
				CMSMstrcat(promptString, CMSM_PARSER_SPACE_MINUS_SPACE_MSG);
				CMSMxtoa(tempString, Max);
				CMSMstrcat(promptString, tempString);

            /* Copy the Default value to defaultString. */
            if (Flags & DEFAULTPRESENT)
               CMSMxtoa(defaultString, *(int *)defaultValue);

				break;

		}
		/* Now add the keyword and : to the prompt. */
		CMSMstrcat(promptString, CMSM_PARSER_NEWLINE_MSG);
		CMSMstrcat(promptString, Keyword);
		CMSMstrcat(promptString, CMSM_PARSER_SPACE_COLON_SPACE_MSG);

	}
#endif
	/* We need to special case the following standard keywords. */
	/* INTPARAM, PORTPARAM, DMAPARAM, MEMPARAM. */
	switch (ParameterType)
	{
		case INTPARAM:
			status = CMSMGetParameter(
                     CMSM_PARSER_INTO_TXTMSG,
                     screenID,
							commandLine,
                     TRUE,
#ifdef OS_NT
                     NULL,
							NULL,
							(UINT16)(Flags & ~REQUIREDPARAM),
#else
                     promptString,
							defaultString,
							Flags & ~REQUIREDPARAM,
#endif
							1,
							param,
							&paramFlags);

			break;
		case PORTPARAM:
			status = CMSMGetParameter(
							CMSM_PARSER_PORT0_TXTMSG,
                     screenID,
							commandLine,
                     TRUE,
#ifdef OS_NT
                     NULL,
							NULL,
							(UINT16)(Flags & ~REQUIREDPARAM),
#else
                     promptString,
							defaultString,
							Flags & ~REQUIREDPARAM,
#endif
							1,
							param,
							&paramFlags);

			break;
		case DMAPARAM:
			status = CMSMGetParameter(
							CMSM_PARSER_DMA0_TXTMSG,
                     screenID,
							commandLine,
                     TRUE,
#ifdef OS_NT
                     NULL,
							NULL,
							(UINT16)(Flags & ~REQUIREDPARAM),
#else
                     promptString,
							defaultString,
							Flags & ~REQUIREDPARAM,
#endif
							1,
							param,
							&paramFlags);

			break;
		case MEMPARAM:
			status = CMSMGetParameter(
							CMSM_PARSER_MEM0_TXTMSG,
                     screenID,
							commandLine,
                     TRUE,
#ifdef OS_NT
                     NULL,
							NULL,
							(UINT16)(Flags & ~REQUIREDPARAM),
#else
                     promptString,
							defaultString,
							Flags & ~REQUIREDPARAM,
#endif
							1,
							param,
							&paramFlags);

			break;
	}

#ifdef OS_NT
	if (status == ODISTAT_ITEM_NOT_PRESENT)
   {
		status = CMSMGetParameter(
						Keyword,
                  screenID,
						commandLine,
                  TRUE,
                  NULL,
						NULL,
						Flags,
						1,
						param,
						&paramFlags);
	}

	/* Now lets check to see if the entered value matches one of ours. */
  	if (status == ODISTAT_SUCCESSFUL)
   {
	   switch (Flags & TYPEMASK)
		{
  			case STRINGTYPE:
   			/* This case is invalid will not get here. */
	   		break;
		   case INTTYPE:
			   /* We need to se if the value is in our range. */
				ivalue = CMSMatoi(param);
  				if (!(ivalue >= Min && ivalue <= Max))
					status = ODISTAT_FAIL;
			   break;
			case UINTTYPE:
  				uivalue = CMSMatoi(param);
   			if (!(uivalue >= (UINT32)Min && uivalue <= (UINT32)Max))
					status = ODISTAT_FAIL;
  				break;
  			case XINTTYPE:
   			uivalue = CMSMatox(param);
            if (uivalue < (UINT32)Min || uivalue > (UINT32)Max)
					status = ODISTAT_FAIL;
  				break;
  		}
   }
#else
	if (status == ODISTAT_ITEM_NOT_PRESENT)
   {
		while (!found)
		{
			status = CMSMGetParameter(
							Keyword,
                     screenID,
							commandLine,
                     TRUE,
                     promptString,
							defaultString,
							Flags,
							1,
							param,
							&paramFlags);

			ForcePrompt = FALSE;

   		/* Now lets check to see if the entered value matches one of ours. */
	   	if (status == ODISTAT_SUCCESSFUL)
		   {
			   switch (Flags & TYPEMASK)
   			{
	   			case STRINGTYPE:
		   			/* This case is invalid will not get here. */
			   		break;
				   case INTTYPE:
					   /* We need to se if the value is in our range. */
   					ivalue = CMSMatoi(param);
	   				if (ivalue >= Min && ivalue <= Max)
		   			{
			   			found = TRUE;
				   	}
						else
						{
							/* The returned value was wrong. */
							/* Set the command line to NULL incase it was bad. */
							ForcePrompt = TRUE;
						}
					   break;
   				case UINTTYPE:
	   				uivalue = CMSMatoi(param);
		   			if (uivalue >= Min && uivalue <= Max)
			   		{
				   		found = TRUE;
					   }
						else
						{
							/* The returned value was wrong. */
							/* Set the command line to NULL incase it was bad. */
							ForcePrompt = TRUE;
						}
   					break;
	   			case XINTTYPE:
		   			uivalue = CMSMatox(param);
			   		if (uivalue >= Min && uivalue <= Max)
				   	{
					   	found = TRUE;
   					}
						else
						{
							/* The returned value was wrong. */
							/* Set the command line to NULL incase it was bad. */
							ForcePrompt = TRUE;

						}
	   				break;
		   	}
   		}
			else /* No parameter was found and it was optional anyway. */
			{
				found = TRUE;	/* Need to get out of the loop. */
			}
      }
	}


	/* The dirverData parameter in the CMSMFree call is NULL in this case */
	/* since the MSM is doing the alloc and free of the memory. */
	CMSMFree(NULL, promptString);
#endif

	return(status);					
}

#ifndef OS_NT
/*======================================================================
  Procedure Name:	CMSMIsValidPtr

  Arguments:	
				  pointer       - void * to be tested.

  Returns:		BOOLEAN

  Abstract:		IsValidPtr takes a purported pointer and returns a BOOLEAN.
               It returns FALSE if using the parameter as a pointer would
               cause an exception or if it can determine some other way
               that the parameter is not a valid pointer.

  Notes:				

=======================================================================*/
BOOLEAN CMSMIsValidPtr(void *pointer)

{
#ifdef SVR
	return(!ValidateAddressRange( pointer, 1, 3));
#endif
#ifdef OS_NIOS
   if (NiosLinToPhys(pointer) == -1)
		return FALSE;
	return TRUE;
#endif
}


/*======================================================================
  Procedure Name:	CMSMPromptUserForInput

  Arguments:	
              screenID          - Screen Handle for user I/O
              prompt            - MEON_STRING * to the prompt string
              defaultValue           - MEON_STRING * to the string value of the
                                   default for this param, NULL if none.
              param             - MEON_STRING * to a buffer where the user's
                                   input is to be returned.

  Returns:		ODISTAT

  Abstract:		This function will display the prompt string (and default
                 value if supplied) and accept and return the user's
                 response.

  Notes:				

=======================================================================*/
ODISTAT CMSMPromptUserForInput(
					SCREEN_HANDLE  *screenID,
					MEON_STRING           *AllowedCharSet,
					MEON_STRING           *prompt,
					MEON_STRING           *defaultValue,
					MEON_STRING           *param,
					UINT32			bufferSize)

{
	UINT32 status;
#ifdef SVR
	SCREEN_HANDLE *usableScreen;
	UINT16 PromptCursorPosition;

	/* Check to see if we can read from the keyboard. */
	if (screenID->flags & KEYBOARD_INPUT_ACTIVE)
	{
		/* We cannot read form the keyboard.  It is not ours to get input from. */
		/* Get an alternate console screen. */
		if ((usableScreen = GetSystemConsoleScreen()) == NULL)
		{
			return ODISTAT_FAIL;
		}

		/* Make sure we can read from the keyboard with this screen. */
		if (usableScreen->flags & KEYBOARD_INPUT_ACTIVE)
		{
			return ODISTAT_FAIL;
		}

		/* Make the screen the one the user can see. */
		ActivateScreen(usableScreen);
	}
	else
	{
		/* The original screen is ok to use.  Set the usable screen to it. */
		usableScreen = screenID;
	}

	OutputToScreen(usableScreen, (char *)CMSM_PARSER_PERCENT_S_MSG, prompt);
	SetInputToOutputCursorPosition(usableScreen);
	EnableInputCursor(usableScreen);
	PromptCursorPosition = usableScreen->inputCursorPosition;
	status = -1;

	*param = '\0';


	status = InputFromKeyboard(
						usableScreen,
						(char *)AllowedCharSet,
						bufferSize,
						bufferSize,
						(char *)param,
						1,
						defaultValue != NULL,
						(char *)defaultValue);

	OutputToScreen(usableScreen,
		(char *)CMSM_PARSER_PERCENT_S_MSG,
		(char *)CMSM_PARSER_NEWLINE_MSG);
   DisableInputCursor(usableScreen);

	if (status != 0)
		return ODISTAT_FAIL;
	return status;
#endif
#ifdef OS_NIOS
   MEON_STRING promptTitle[80];	

	screenID = screenID;
	AllowedCharSet = AllowedCharSet;

	NiosSprintf(promptTitle,
					(UINT8 *)CMSM_PARSER_COMMAND_PROMPT_MSG,
					CurrentConfigTable->MLIDCFG_ShortName);

   CMSMstrcpy( param, defaultValue );
	status = NiosVidInputDialogBox(
						(UINT8 *)promptTitle,
						(UINT8 *)prompt,
						(UINT8 *)param,
						bufferSize,
                  0);
	if (status == IDOK)
	{
		return ODISTAT_SUCCESSFUL;
	}
	return ODISTAT_FAIL;
#endif  /* OS_NIOS */
}



ODISTAT  CMSMCheckForHardwareConflict(
            UINT32   value,
  				UINT16	ParameterType)
{
   ODISTAT  conflict;
   /* We need to special case the following standard keywords. */
   /* INTPARAM, PORTPARAM, DMAPARAM, MEMPARAM. */
   switch (ParameterType)
   {
   	case INTPARAM:
   	case INT1PARAM:
         conflict = CMSMCheckInterrupt(
                     value,
		               CurrentConfigTable->MLIDCFG_SharingFlags);
            			break;
  		case PORTPARAM:
         conflict = CMSMCheckIOPort(
                     value,
                     CurrentConfigTable->MLIDCFG_IORange0,
                     CurrentConfigTable->MLIDCFG_SharingFlags);
  			break;
  		case PORT1PARAM:
         conflict = CMSMCheckIOPort(
                     value,
                     CurrentConfigTable->MLIDCFG_IORange1,
                     CurrentConfigTable->MLIDCFG_SharingFlags);
  			break;
 		case DMAPARAM:
  		case DMA1PARAM:
         conflict = CMSMCheckDMA(
                     value,
                     CurrentConfigTable->MLIDCFG_SharingFlags);
  			break;
 		case MEMPARAM:
         conflict = CMSMCheckMemoryDecode(
                     value,
                     (UINT32)CurrentConfigTable->MLIDCFG_MemoryAddress0,
                     CurrentConfigTable->MLIDCFG_SharingFlags);
  			break;
 		case MEM1PARAM:
         conflict = CMSMCheckMemoryDecode(
                     value,
                     (UINT32)CurrentConfigTable->MLIDCFG_MemoryAddress1,
                     CurrentConfigTable->MLIDCFG_SharingFlags);
 			break;
      case SLOTPARAM:
         conflict = CMSMCheckSlot(value);
         break;
      default:
         conflict = ODISTAT_SUCCESSFUL;
         break;
 	}
   return conflict;
}
#endif

/*======================================================================
  Procedure Name:	CMSMASCIINodeToHEX

  Arguments:	
              configTable       - Pointer to the config Table
              asciiNode         - Pointer to the ASCII Node value.

  Returns:		ODISTAT

  Abstract:		This function will convert an ASCII node address to a hex node
               Address.

  Notes:				

=======================================================================*/
ODISTAT  CMSMASCIINodeToHex(
            CONFIG_TABLE   *configTable,
            MEON_STRING    *asciiNode)
{
   FRAME_DATA  *frameData;
   UINT32      length, i;
   MEON        nodeChar;


   frameData = CONFIG_TO_MSM (configTable);

   /* Lets make sure the string is in upper case. */
   CMSMstrupr(asciiNode);

   length = CMSMstrlen(asciiNode);

	/* Check to see if an M or L was entered. */
	if (asciiNode[length -1] == 'M' || asciiNode[length -1] == 'L')
		--length;

	if (length <= ADDR_SIZE * 2)
		i = (ADDR_SIZE * 2) - length;
	else i = 0;

	/* Zero out the upper part of the address if not entered. */
   for (length = 0; length < i; ++length)
		configTable->MLIDCFG_NodeAddress.nodeAddress[length/2] &= 0x0f << (4 * (length & 1));

	length = 0;

	for (; i < ADDR_SIZE * 2; ++i, ++length)
	{
		if (asciiNode[length] >= '0' && asciiNode[length] <= '9')
		{
			nodeChar = asciiNode[length] - '0';
		}
		else
		{
			if (asciiNode[length] >= 'A' && asciiNode[length] <= 'F')
				nodeChar = asciiNode[length] - 'A' + 10;
			else
			{
				/* We have a problem with the node address override. */
#ifdef OS_NT
				CMSMLogEvent(
								MSG_TYPE_INIT_WARNING,
								BAD_NODE_ADDRESS,
                        configTable->MLIDCFG_ShortName);
#else
				CMSMPrintString(
								configTable,
								MSG_TYPE_INIT_WARNING,
                        CMSM_PARSER_NODE_ADDR_ERROR_MSG,
                        configTable->MLIDCFG_ShortName,
								(VOID *)83);
#endif


				COPY_ADDR(configTable->MLIDCFG_NodeAddress.nodeAddress, BroadcastAddress);
            return (ODISTAT_FAIL);
			}
		}
		if (i & 1)
		{
			configTable->MLIDCFG_NodeAddress.nodeAddress[i/2] &= nodeChar | 0xf0;
		}
      else configTable->MLIDCFG_NodeAddress.nodeAddress[i/2] &= (nodeChar << 4) | 0x0f;
	}

	/* We need to call the Media override routine. */
#ifndef NWCACS
	frameData->MPBlock->MediaNodeOverridePtr(
					frameData,
					asciiNode[length]);
#else
  CACSMediaNodeOverride( frameData, (UINT32)asciiNode[length] ) ;
#endif

   return (ODISTAT_SUCCESSFUL);

} /* End CMSMASCIINodeToHex */

#ifndef OS_NT
/****************************************************************************/

/*	Check to see if the two memory areas submitted overlap.  If overlap is
	detected, return TRUE.  If no overlap is detected, return FALSE. */

ODISTAT CMSMCheckRange(
		UINT32 Base0,
		UINT32 Range0,
		UINT32 Base1,
		UINT32 Range1)
{
	if ((Range0 == 0) || (Range1 == 0))
		return(ODISTAT_SUCCESSFUL);

	if (Base0 < Base1)
	{
		if ((Base0 + Range0) > Base1)
			return(ODISTAT_DUPLICATE_ENTRY);
	}
	else
	{
		if ((Base1 + Range1) > Base0)
			return(ODISTAT_DUPLICATE_ENTRY);
	}

	return(ODISTAT_SUCCESSFUL);
}


/****************************************************************************/

ODISTAT  CMSMCheckValue(
		UINT8 Value0,
		UINT8 Value1)
{

	if (Value0 == 0xFF)
		return(ODISTAT_SUCCESSFUL);

	if (Value0 == Value1)
		return(ODISTAT_DUPLICATE_ENTRY);

	return(ODISTAT_SUCCESSFUL);
}


/****************************************************************************/

ODISTAT CMSMCheckIOPort(
		UINT32 IOPort,
		UINT32 IOLength,
		UINT8  ShareFlag)
{
	IO_CONFIG   *WorkIOConfig;

	for (WorkIOConfig = IOConfigurationList; WorkIOConfig != NULL;
			WorkIOConfig = WorkIOConfig->IO_DriverLink)
	{
		if (CMSMCheckRange(IOPort, IOLength, WorkIOConfig->IO_IOPort0,
				WorkIOConfig->IO_IORange0))
		{
			if (((WorkIOConfig->IO_SharingFlags & MS_SHARE_PORT0_BIT) == 0) ||
					(ShareFlag == 0))
				return (ODISTAT_DUPLICATE_ENTRY);
		}
		if (CMSMCheckRange(IOPort, IOLength, WorkIOConfig->IO_IOPort1,
				WorkIOConfig->IO_IORange1))
		{
			if (((WorkIOConfig->IO_SharingFlags & MS_SHARE_PORT1_BIT) == 0) ||
					(ShareFlag == 0))
				return (ODISTAT_DUPLICATE_ENTRY);
		}
	}

	return (ODISTAT_SUCCESSFUL);
}


/****************************************************************************/

/* 	See if the submitted address overlaps with an already registered section
	of memory.  If it does, check to see if sharing is allowed.  Return 0 if
	no conflicts are found.  Return 1 if the submitted memory overlaps with
	registered memory AND no sharing is permitted */

ODISTAT  CMSMCheckMemoryDecode(
		UINT32   MemoryAddress,
		UINT32   MemoryLength,
		UINT8    ShareFlag)
{
	IO_CONFIG   *WorkIOConfig;

	for (WorkIOConfig = IOConfigurationList;WorkIOConfig != NULL;WorkIOConfig = WorkIOConfig->IO_DriverLink)
	{
		if (CMSMCheckRange(MemoryAddress, MemoryLength << 4,
			(UINT32)WorkIOConfig->IO_MemoryAddress0,
			WorkIOConfig->IO_MemorySize0 << 4))
		{
			if (((WorkIOConfig->IO_SharingFlags & MS_SHARE_MEMORY0_BIT) == 0) || (ShareFlag == 0))
				return (ODISTAT_DUPLICATE_ENTRY);
		}
		if (CMSMCheckRange(MemoryAddress, MemoryLength << 4,
			(UINT32)WorkIOConfig->IO_MemoryAddress1,
			WorkIOConfig->IO_MemorySize0 << 4))
		{
			if (((WorkIOConfig->IO_SharingFlags & MS_SHARE_MEMORY1_BIT) == 0) || (ShareFlag == 0))
				return (ODISTAT_DUPLICATE_ENTRY);
		}
	}

	return (ODISTAT_SUCCESSFUL);
}


/****************************************************************************/

ODISTAT CMSMCheckInterrupt(
		UINT32   InterruptNumber,
		UINT8    ShareFlag)
{
	IO_CONFIG   *WorkIOConfig;

	for (WorkIOConfig = IOConfigurationList; WorkIOConfig != NULL;
			WorkIOConfig = WorkIOConfig->IO_DriverLink)
	{
		if (CMSMCheckValue(InterruptNumber, WorkIOConfig->IO_Interrupt0))
		{
			if (((WorkIOConfig->IO_SharingFlags & MS_SHARE_IRQ0_BIT) == 0) ||
					(ShareFlag == 0))
				return (ODISTAT_DUPLICATE_ENTRY);
		}
		if (CMSMCheckValue(InterruptNumber, WorkIOConfig->IO_Interrupt1))
		{
			if (((WorkIOConfig->IO_SharingFlags & MS_SHARE_IRQ1_BIT) == 0) ||
					(ShareFlag == 0))
				return (ODISTAT_DUPLICATE_ENTRY);
		}
	}

	return (ODISTAT_SUCCESSFUL);
}


/****************************************************************************/

ODISTAT CMSMCheckSlot(
		UINT32 SlotNumber)
{
	IO_CONFIG   *WorkIOConfig;

	if (SlotNumber == 0)
		return (ODISTAT_SUCCESSFUL);

	for (WorkIOConfig = IOConfigurationList; WorkIOConfig != NULL;
			WorkIOConfig = WorkIOConfig->IO_DriverLink)
		if (SlotNumber == WorkIOConfig->IO_Slot)
				return (ODISTAT_DUPLICATE_ENTRY);

	return (ODISTAT_SUCCESSFUL);
}

/****************************************************************************/

ODISTAT  CMSMCheckDMA(
		UINT32   DMALevel,
		UINT8    ShareFlag)
{
	IO_CONFIG   *WorkIOConfig;

	for (WorkIOConfig = IOConfigurationList;WorkIOConfig != NULL;WorkIOConfig = WorkIOConfig->IO_DriverLink)
	{
		if (CMSMCheckValue(DMALevel, WorkIOConfig->IO_DMALine0))
		{
			if (((WorkIOConfig->IO_SharingFlags & MS_SHARE_DMA0_BIT) == 0) || (ShareFlag == 0))
				return (ODISTAT_DUPLICATE_ENTRY);
		}
		if (CMSMCheckValue(DMALevel, WorkIOConfig->IO_DMALine1))
		{
			if (((WorkIOConfig->IO_SharingFlags & MS_SHARE_DMA1_BIT) == 0) || (ShareFlag == 0))
				return (ODISTAT_DUPLICATE_ENTRY);
		}
	}

	return (ODISTAT_SUCCESSFUL);
}
#endif

#if defined( IAPX386 ) && !defined( NWCACS )
/* CMSMOldParseDriverParameters and CMSMParseCustomKeyowrds are only called */
/* from Intel assembly lan drivers. */
/***********************************************************************\
*                                                                       *
*   Name:
*		ODISTAT    CMSMOldParseDriverParameters(
*			CONFIG_TABLE	*configTable,
*			HSM_OPTIONS		*adapterOptions,
*			UINT32			needsIOBits);
*
*   Description:
*		This routine is called by MSMParseDriverParameters to map the
*		old parsing method which used ParseDriverParameters found in NIOS
*		to CMSMParseDriverParameters.
*
*   Parameters in:
*		configTable     -   Pointer to HSM configuration table.
*		adapterOptions  -   Pointer to Adapter Options Structure.
*		NeedsIOBit      -   resources to be parsed for(bit map).
*
*   Values returned:
*		ODISTAT_SUCCESSFUL	Successful new hardware entry.
*		ODISTAT_FAIL			Parsing of parameters was not successful.
*
*   Implied parameters:
*      NONE
*
*   Notes:
*
************************************************************************/
ODISTAT _cdecl
CMSMOldParseDriverParameters (
            CONFIG_TABLE		*configTable,
            HSM_OPTIONS			*adapterOptions,
            UINT32				needsIOBits)
{
	FRAME_DATA			*frameData;
	DRIVER_OPTION		*DriverOptionsListHead;
	DRIVER_OPTION		*DriverOptionsListTail;
	DRIVER_OPTION		*DriverOptionNode;
	DRIVER_PARM_BLOCK	*driverParmBlock;
	UINT32				retValue, UnableToGetMem, i;

	/* Set up Local Variables. */

	frameData = CONFIG_TO_MSM (configTable);
	driverParmBlock = frameData->DPBlock;
	CMSMInitParser(driverParmBlock);
	DriverOptionsListHead = NULL;
	DriverOptionsListTail = NULL;
	UnableToGetMem = 0;

	/* Need to set CommandLine for other parse routines	*/

	CommandLine = frameData->DPBlock->DriverInitParmPointer->CommandLine;

	
	/* Build driverOption list */

	/* Check for IOSlot and build DRIVER_OPTION node if present. */

	if(adapterOptions->IOSlot)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}

		else
		{
			DriverOptionNode->Link = NULL;

			/* Add to List */

			DriverOptionsListHead = DriverOptionNode;
			DriverOptionsListTail = DriverOptionsListHead;


			/* Fill in DriverOption structure */

			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr
				= (void *)adapterOptions->IOSlot;
			DriverOptionNode->Parameter1.Range = 0;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = SLOTPARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */

			if(needsIOBits & NEEDS_SLOT)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}
		}
	}

	/* Check for IOPort0 and build DRIVER_OPTION node if present. */

	if(adapterOptions->IOPort0 && UnableToGetMem == 0)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}

		else
		{
			DriverOptionNode->Link = NULL;

			/* Add to List */

			if (!DriverOptionsListHead)
			{
				DriverOptionsListHead = DriverOptionNode;
				DriverOptionsListTail = DriverOptionsListHead;
			}

			else
			{
				DriverOptionsListTail->Link = DriverOptionNode;
				DriverOptionsListTail = DriverOptionNode;
			}

			/* Fill in DriverOption structure */

			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr
				= (void *)adapterOptions->IOPort0;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = PORTPARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */

			if(needsIOBits & NEEDS_PORT0)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}

			DriverOptionNode->Parameter1.Range = 0;

			/* See if there is a length */

			if(adapterOptions->IOLength0)
			{
				/* Find the largest length and use it for the range */

				for (i = 0; i <= (UINT32)adapterOptions->IOLength0->NumberOfOptions;
					i++)
				{
					if (DriverOptionNode->Parameter1.Range
						< (UINT32)adapterOptions->IOLength0->OptionData[i])
					{
						DriverOptionNode->Parameter1.Range
							= (UINT32)adapterOptions->IOLength0->OptionData[i];
					}
				}
			}
		}
	}

	/* Check for IOPort1 and build DRIVER_OPTION node if present. */

	if(adapterOptions->IOPort1 && UnableToGetMem == 0)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}

		else
		{
			DriverOptionNode->Link = NULL;

			/* Add to List */

			if (!DriverOptionsListHead)
			{
				DriverOptionsListHead = DriverOptionNode;
				DriverOptionsListTail = DriverOptionsListHead;
			}

			else
			{
				DriverOptionsListTail->Link = DriverOptionNode;
				DriverOptionsListTail = DriverOptionNode;
			}


			/* Fill in DriverOption structure */

			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr
				= (void *)adapterOptions->IOPort1;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = PORT1PARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */

			if(needsIOBits & NEEDS_PORT1)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}

			DriverOptionNode->Parameter1.Range = 0;

			/* See if there is a length */

			if(adapterOptions->IOLength1)
			{
				/* Find the largest length and use it for the range */

				for (i = 0; i <= (UINT32)adapterOptions->IOLength1->NumberOfOptions;
					i++)
				{
					if (DriverOptionNode->Parameter1.Range
						< (UINT32)adapterOptions->IOLength1->OptionData[i])
					{
						DriverOptionNode->Parameter1.Range
							= (UINT32)adapterOptions->IOLength1->OptionData[i];
					}
				}
			}
		}
	}

	/* Check for MemoryDecode0 and build DRIVER_OPTION node if present. */

	if(adapterOptions->MemoryDecode0 && UnableToGetMem == 0)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}

		else
		{
			DriverOptionNode->Link = NULL;

			/* Add to List */

			if (!DriverOptionsListHead)
			{
				DriverOptionsListHead = DriverOptionNode;
				DriverOptionsListTail = DriverOptionsListHead;
			}

			else
			{
				DriverOptionsListTail->Link = DriverOptionNode;
				DriverOptionsListTail = DriverOptionNode;
			}

			/* Fill in DriverOption structure */

			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr
				= (void *)adapterOptions->MemoryDecode0;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = MEMPARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */

			if(needsIOBits & NEEDS_MEMORY0)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}

			DriverOptionNode->Parameter1.Range = 0;

			/* See if there is a range */

			if(adapterOptions->MemoryLength0)
			{
				/* Find the largest length and use it for the range */

				for (i = 0;
					i <= (UINT32)adapterOptions->MemoryLength0->NumberOfOptions; i++)
				{
					if (DriverOptionNode->Parameter1.Range
						< (UINT32)adapterOptions->MemoryLength0->OptionData[i])
					{
						DriverOptionNode->Parameter1.Range
							= (UINT32)adapterOptions->MemoryLength0->OptionData[i];
					}
				}
			}
		}
	}

	/* Check for MemoryDecode1 and build DRIVER_OPTION node if present. */

	if(adapterOptions->MemoryDecode1 && UnableToGetMem == 0)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}

		else
		{

			DriverOptionNode->Link = NULL;

			/* Add to List */

			if (!DriverOptionsListHead)
			{
				DriverOptionsListHead = DriverOptionNode;
				DriverOptionsListTail = DriverOptionsListHead;
			}

			else
			{
				DriverOptionsListTail->Link = DriverOptionNode;
				DriverOptionsListTail = DriverOptionNode;
			}

			/* Fill in DriverOption structure */

			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr
				= (void *)adapterOptions->MemoryDecode1;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = MEM1PARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */

			if(needsIOBits & NEEDS_MEMORY1)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}

			DriverOptionNode->Parameter1.Range = 0;

			/* See if there is a length */

			if(adapterOptions->MemoryLength1)
			{
				/* Find the largest length and use it for the range */

				for (i = 0;
					i <= (UINT32)adapterOptions->MemoryLength1->NumberOfOptions; i++)
				{
					if (DriverOptionNode->Parameter1.Range
						< (UINT32)adapterOptions->MemoryLength1->OptionData[i])
					{
						DriverOptionNode->Parameter1.Range
							= (UINT32)adapterOptions->MemoryLength1->OptionData[i];
					}
				}
			}
		}
	}

	/* Check for Interrupt0 and build DRIVER_OPTION node if present. */

	if(adapterOptions->Interrupt0 && UnableToGetMem == 0)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}

		else
		{
			DriverOptionNode->Link = NULL;

			/* Add to List */

			if (!DriverOptionsListHead)
			{
				DriverOptionsListHead = DriverOptionNode;
				DriverOptionsListTail = DriverOptionsListHead;
			}

			else
			{
				DriverOptionsListTail->Link = DriverOptionNode;
				DriverOptionsListTail = DriverOptionNode;
			}

			/* Fill in DriverOption structure */

			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr
				= (void *)adapterOptions->Interrupt0;
			DriverOptionNode->Parameter1.Range = 0;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = INTPARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */

			if(needsIOBits & NEEDS_INTERRUPT0)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}
		}
	}

	/* Check for Interrupt1 and build DRIVER_OPTION node if present. */

	if(adapterOptions->Interrupt1 && UnableToGetMem == 0)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}

		else
		{
			DriverOptionNode->Link = NULL;

			/* Add to List */

			if (!DriverOptionsListHead)
			{
				DriverOptionsListHead = DriverOptionNode;
				DriverOptionsListTail = DriverOptionsListHead;
			}

			else
			{
				DriverOptionsListTail->Link = DriverOptionNode;
				DriverOptionsListTail = DriverOptionNode;
			}

			/* Fill in DriverOption structure */

			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr
				= (void *)adapterOptions->Interrupt1;
			DriverOptionNode->Parameter1.Range = 0;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = INT1PARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */

			if(needsIOBits & NEEDS_INTERRUPT1)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}
		}
	}

	/* Check for DMA0 and build DRIVER_OPTION node if present. */

	if(adapterOptions->DMA0 && UnableToGetMem == 0)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}

		else
		{
			DriverOptionNode->Link = NULL;

			/* Add to List */

			if (!DriverOptionsListHead)
			{
				DriverOptionsListHead = DriverOptionNode;
				DriverOptionsListTail = DriverOptionsListHead;
			}

			else
			{
				DriverOptionsListTail->Link = DriverOptionNode;
				DriverOptionsListTail = DriverOptionNode;
			}

			/* Fill in DriverOption structure */

			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr = (void *)adapterOptions->DMA0;
			DriverOptionNode->Parameter1.Range = 0;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = DMAPARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */

			if(needsIOBits & NEEDS_DMA0)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}
		}
	}

	/* Check for DMA1 and build DRIVER_OPTION node if present. */

	if(adapterOptions->DMA1 && UnableToGetMem == 0)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}

		else
		{
			DriverOptionNode->Link = NULL;

			/* Add to List */

			if (!DriverOptionsListHead)
			{
				DriverOptionsListHead = DriverOptionNode;
				DriverOptionsListTail = DriverOptionsListHead;
			}

			else
			{
				DriverOptionsListTail->Link = DriverOptionNode;
				DriverOptionsListTail = DriverOptionNode;
			}

			/* Fill in DriverOption structure */

			DriverOptionNode->Link = NULL;
			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr = (void *)adapterOptions->DMA1;
			DriverOptionNode->Parameter1.Range = 0;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = DMA1PARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */
		
			if(needsIOBits & NEEDS_DMA1)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}
		}
	}

	/* Check for Channel and build DRIVER_OPTION node if present. */

	if(adapterOptions->Channel && UnableToGetMem == 0)
	{
		/* Allocate memory DRIVER_OPTION node. */

		DriverOptionNode = CMSMInitAlloc(sizeof(struct _DRIVER_OPTION_));

		if (!DriverOptionNode)
		{
			UnableToGetMem++;
		}
		else
		{
			DriverOptionNode->Link = NULL;

			/* Add to List */

			if (!DriverOptionsListHead)
			{
				DriverOptionsListHead = DriverOptionNode;
				DriverOptionsListTail = DriverOptionsListHead;
			}

			else
			{
				DriverOptionsListTail->Link = DriverOptionNode;
				DriverOptionsListTail = DriverOptionNode;
			}

			/* Fill in DriverOption structure */

			DriverOptionNode->ParseString = NULL;
			DriverOptionNode->Parameter0.OptionPtr
				= (void *)adapterOptions->Channel;
			DriverOptionNode->Parameter1.Range = 0;
			DriverOptionNode->Parameter2.StringDefault = NULL;
			DriverOptionNode->Type = CHANNELPARAM;
			DriverOptionNode->String[0] = '\0';

			/* Check DriverNeedsBitMask and set Flags to correspond */

			if(needsIOBits & NEEDS_CHANNEL)
			{
				DriverOptionNode->Flags = REQUIREDPARAM | ENUMPARAM;
			}

			else
			{
				DriverOptionNode->Flags = OPTIONALPARAM | ENUMPARAM;
			}
		}
	}

	if (UnableToGetMem)
	{
		parserInitialized = FALSE;	
		retValue = ODISTAT_FAIL;
	}
	else
	{
		retValue = CMSMParseDriverParameters(driverParmBlock,
			DriverOptionsListHead);

		/* Map return value for optional parameter not found into successful. */

		if(retValue == ODISTAT_SUCCESSFUL || retValue == ODISTAT_ITEM_NOT_PRESENT)
		{
			retValue = CMSMParseCustomKeywords(frameData->DPBlock, configTable,
							frameData->DPBlock->DriverInitParmPointer->CommandLine);
		}
	}

	/* Free up memory before leaving. */

	while (DriverOptionsListHead)
	{
		DriverOptionNode = DriverOptionsListHead;
		DriverOptionsListHead = DriverOptionsListHead->Link;
		CMSMFree(NULL, DriverOptionNode);
	};

   return (retValue);

}   /* End CMSMOldParseDriverParameters */

/*++======================================================================
//  Procedure Name:	CMSMParseCustomKeywords
//								IN		DRIVER_PARM_BLOCK	*parameters
//                      IN    CONFIG_TABLE		*configTable
//                      IN    MEON_STRING       *commandLine
//
//  Arguments:			parameters        -  Pointer to HSM parameter Block.
//                   configTable       -  Pointer to the Config Table.
//                   commandLine       _  Pointer to the Load Options.
//
//  Returns:		ODISTAT
//							ODISTAT_SUCCESSFUL
//							ODISTAT_FAIL
//
//  Abstract:			This routine will scan the Load Options (Command Line)
//                   for the defined custom keywords.  It will then convert
//                   the string value to the defined type, and call the
//                   custom routine.
//
//  Notes:				
//
//=====================================================================--*/
ODISTAT _cdecl
CMSMParseCustomKeywords(
				DRIVER_PARM_BLOCK	*parameters,
				CONFIG_TABLE		*configTable,
				MEON_STRING		   *commandLine)
{
   UINT32      i, k, done, numToCopy;
	UINT32	   numKeywords,
               keywordInputType,
               value,
					keywordRequired,
  				   keywordTextLen;
   ODISTAT	   status = ODISTAT_SUCCESSFUL;
	PMEON 	   keywordText, keyword;
	MEON_STRING tempBuffer[MAX_PARAM_LEN];
	MEON_STRING keywordString[MAX_PARAM_LEN];
	MEON_STRING tempKeyword[6];
	UINT32	   (_cdecl *keywordRoutine)(MLID_CONFIG_TABLE *, PUINT8, UINT32);
	ODISTAT		promptStatus;
	MEON			*keywordDefaultStr;
	MEON			*keywordPromptStr;
	MEON_STRING		*separators = CMSM_PARSER_WHITESPACE_EQUAL_SEPARATORS_MSG;
	MEON_STRING		*BadKeyword = CMSM_PARSER_KEYWORD_OUT_RANGE_MSG;
	ASM_DRIVER_KEYWORD_STRUCT *keywordStruct;


	/*Get the Number of Custom Keywords*/
	keywordStruct = (ASM_DRIVER_KEYWORD_STRUCT *)&parameters->DPB_Reserved1;
	numKeywords = keywordStruct->DriverNumKeywords;

	/* Need to set CommandLine for other parse routines	*/
	CommandLine = commandLine;

#ifdef OS_NT
	/* The command line is single NULL separated per parameter. */
	/* Remove the single NULL separaters from the command line */
	/* and replace it with a space so that other APIs will see */
	/* a single null terminated string. */
   for (i = 0; *(UINT16 *)&commandLine[i] != 0; ++i)
	{
      if (commandLine[i] == '\0')
		{
         commandLine[i] = ' ';
		}
	}
#endif

  	/* We need to UPCASE the command LINE. */
 	CMSMstrupr(commandLine);

	/*Check for the keywords on the command line.*/
   for (i = 0; i < numKeywords && status != ODISTAT_FAIL; ++i)
	{
		keywordText = keywordStruct->DriverKeywordText[i];
		keywordTextLen = keywordStruct->DriverKeywordTextLen[i] & T_LENGTH_MASK;
		keywordInputType = keywordStruct->DriverKeywordTextLen[i] & T_TYPE_MASK;
		keywordRequired = keywordStruct->DriverKeywordTextLen[i] & T_REQUIRED;
		keywordRoutine = keywordStruct->DriverProcesskeywordTab[i];
		CMSMstrncpy(keywordString, keywordText, keywordTextLen);
		keywordString[keywordTextLen] = '\0';

		/* We need to UPCASE the Keyword. */
		CMSMstrupr(keywordString);

      keyword = CMSMParseStringForKeyword( commandLine, keywordString);


		/* If we have a keyword or it is required, process it. */
      if (keyword != NULL || keywordRequired)
		{

			/*Check for String.*/
			if (keywordInputType == T_STRING)
			{
				if (keyword == NULL && keywordRequired)
				{
					status = ODISTAT_FAIL;
				}
			}
			else if ((keywordInputType == T_NUMBER)
				|| (keywordInputType == T_HEX_NUMBER))
			{
				keywordDefaultStr = (MEON *)
					(*(UINT32 *)&keywordText[keywordTextLen + 8]);
				keywordPromptStr = (MEON *)
					(*(UINT32 *)&keywordText[keywordTextLen + 16]);
				done = 0;
				while (!done) /* Loop until the parameter is in range. */
				{
					if (keyword == NULL && keywordRequired)
					{
#ifdef OS_NT
						status = ODISTAT_FAIL;
						done = 1;
#else
						/* CMSMstrcpy( tempBuffer, keywordDefaultStr); */
						if (keywordInputType == T_NUMBER)
						{
							promptStatus = CMSMPromptUserForInput(
								CMSMScreenID,
								NULL,
								keywordPromptStr,
								keywordDefaultStr,
								tempBuffer,
								11);
						}
						else
						{
							promptStatus = CMSMPromptUserForInput(
								CMSMScreenID,
								NULL,
								keywordPromptStr,
								keywordDefaultStr,
								tempBuffer,
								11);
						}
                  if (promptStatus != ODISTAT_SUCCESSFUL)
						{
							status = ODISTAT_FAIL;
							done = 1;
						}
	  					keyword = tempBuffer; /* setup to call asm routine. */
#endif
					}
					else
					{
						if (!keywordRequired)
							done = 1;

						/* advance past the keyword */
						keyword += keywordTextLen + 1;
                  /* Eat white space after the keyword. */
						keyword += CMSMstrspn(keyword, separators);
						numToCopy = CMSMstrcspn(keyword, separators);
						CMSMstrncpy(tempBuffer, keyword, numToCopy);
						tempBuffer[numToCopy] = '\0';
					}

					if (status == ODISTAT_SUCCESSFUL)
					{
						if (keywordInputType == T_NUMBER)
						{
							value = CMSMatoi(tempBuffer);
						}
						else
						{
                     value = CMSMatox(tempBuffer);
						}
				
						/*Check to see if value is in Range.*/
						if ((value > *(UINT32 *)&keywordText[keywordTextLen + 4])
							|| (value < *(UINT32 *)&keywordText[keywordTextLen]))
						{
							/*The value is out of Range*/
#ifdef OS_NT
							done = 1;
							status = ODISTAT_FAIL;
#else
							CMSMPrintString(
									configTable,
									MSG_TYPE_INIT_ERROR,
									BadKeyword,
									NULL,
									NULL);

							if (!keywordRequired)
							{
								/* If not required and value is wrong, fail. */
								status = ODISTAT_FAIL;
							}
#endif
							keyword = NULL;
						}
						else
						{
							done = 1;
						}
					}
				} /* while */
			}
			else  if (keywordInputType == T_HEX_STRING)
			{
				keywordDefaultStr = (MEON *)
					(*(UINT32 *)&keywordText[keywordTextLen + 8]);
				keywordPromptStr = (MEON *)
					(*(UINT32 *)&keywordText[keywordTextLen + 12]);
				done = 0;
				while (!done) /* Loop until the parameter is in range. */
				{
					if (keyword == NULL && keywordRequired)
					{
#ifdef OS_NT
						status = ODISTAT_FAIL;
						done = 1;
#else
						CMSMstrcpy( tempBuffer, keywordDefaultStr);
						promptStatus = CMSMPromptUserForInput(
							CMSMScreenID,
							NULL,
							keywordPromptStr,
							keywordDefaultStr,
							tempBuffer,
							13);
						if (promptStatus == ODISTAT_SUCCESSFUL)
						{
							keyword = tempKeyword;
							numToCopy = CMSMstrlen(tempBuffer);
						}
                  else
						{
							status = ODISTAT_FAIL;
							done = 1;
						}
#endif
					}
					else
					{
						if (!keywordRequired)
							done = 1;
						/*Convert the value to a HEX string*/
						keyword += keywordTextLen + 1;
						/* Eat white space after the keyword. */
						keyword += CMSMstrspn(keyword, separators);
						numToCopy = CMSMstrcspn(keyword, separators);
						CMSMstrncpy(tempBuffer, keyword, numToCopy);
						tempBuffer[numToCopy] = '\0';
						keyword = tempKeyword;
					}

					if (status == ODISTAT_SUCCESSFUL)
					{
						if (numToCopy > 12)
						{
#ifdef OS_NT
							status = ODISTAT_FAIL;
							done = 1;
#else
							/*The string is to long */
							CMSMPrintString(
									configTable,
									MSG_TYPE_INIT_ERROR,
									BadKeyword,
									NULL,
									NULL);

							if (!keywordRequired)
							{
								/* If not required and value is wrong, fail. */
								status = ODISTAT_FAIL;
							}
							keyword = NULL;
							break;
#endif
						}
						else
						{
							done = 1;
						}
					}
				}

#ifdef OS_NT
				/* Always successful in the prompting case. */
				if (status == ODISTAT_SUCCESSFUL)
				{
#endif
					/*Zero out the first 6 bytes of keyword*/
					for (k = 0; k < 6; ++k)
					{
						keyword[k] = 0;
					}

					k = 5;

					while (numToCopy >= 2)
					{
						numToCopy -= 2;

						value = CMSMatox(&tempBuffer[numToCopy]);

						tempBuffer[numToCopy] = '\0';
						keyword[k] = (UINT8)value;
						--k;
					}

					if (numToCopy)	 	/* Get remaining value if numToCopy is ODD	*/
					{
						keyword[k] = (UINT8)CMSMatox(tempBuffer);
					}
#ifdef OS_NT
				}
#endif
			}

			/*Call the Routine.  The routine is in Assembly*/
			if (status == ODISTAT_SUCCESSFUL)
			{
				AsmCustomKeywordRoutine(
							configTable,
							keyword,
							value,
							keywordRoutine);
			}
		} /* The keyword was not present and was not required. */
#ifdef OS_NT
		if (status == ODISTAT_FAIL)
		{
			/*A Required Parameter was left out.*/
			CMSMLogEvent(
						MSG_TYPE_INIT_ERROR,
						BAD_KEYWORD,
						NULL);
		}
#endif
	} /* for */
	return status;

}  /*End CMSMParseCustomKeywords */
#endif

/*++======================================================================
//  Procedure Name:	CMSMParseStringForKeyword
//								IN    MEON    *searchString
//                      IN    MEON    *keyword
//
//  Arguments:			searchString		-	String to search.
//							keyword				-	keyword to find in search string.
//
//  Returns:			MEON *     -   Pointer to where the keyword is in the
//												searchString.
//
//  Abstract:	Thie routine will take a string and search it for the specified
//             keyword.  If found a pointer to that keyword and its data will
//             be returned.
//
//  Notes:				
//
//=====================================================================--*/
MEON *
CMSMParseStringForKeyword(MEON *searchString, MEON *keyword)
{
	MEON *s;
	MEON_STRING		*separators = CMSM_PARSER_WHITESPACE_EQUAL_SEPARATORS_MSG;
	UINT32 len;

	s = searchString;
	/* Find the keyword in the command line. */
   if (s != NULL)
	{

		while (s != NULL)
		{
         s = CMSMstrstr(s, keyword);
			if (s== NULL)
				break;
			/* Now lets make sure this is the keyword and not part of a */
			/* different word. */
			len = CMSMstrlen(keyword);
         if ((CMSMstrcspn(&s[len], separators) == 0) || s[len] == '\0')
			{
            /* If s is at the beginning of the searchString or there is a */
				/* separator preceeding s, then we matched. */
            if (s == searchString || (CMSMstrchr(separators, *(s-1)) != NULL))
				{
					break;
				}
			}
			s += len;
		}
	}
	return s;
}

#ifndef OS_NT
/*++======================================================================
//  Procedure Name:	CMSMCheckDuplicateEntry
//								IN    void    *value
//                      IN    UINT16  parameterType
//
//  Arguments:			value					-	option value to compare.
//							parameterType		-	0x0000 if custom, otherwise one bit
//														set which defines which std param.
//
//  Returns:			UINT32     -   returns board number of previously loaded
//												board which has a matching hardware
//												option = value; OR -1 if no match.
//												
//  Abstract:	This routine will compare the passed in option value to the
//					hardware option values from config table of previously loaded
//					boards. ParameterType defines which type option (INT, PORT,
//					DMA, etc. ) we should compare against. If there is a match
//					we return the matching board number, otherwise return -1.
//					We are only concerned with boards associated with the
//					driver currently being loaded.
//
//  Notes:	We are only concerned with boards associated with the
//				driver currently being loaded.
//
//=====================================================================--*/
UINT32 CMSMCheckDuplicateEntry(void *value, UINT16 parameterType)
{
	UINT32		i;
	FRAME_DATA	*framePtr;
	UINT32		retValue = -1;

	for (i = 0; i < LSLConfigTableptr->LMaxNumberOfBoards; ++i)
	{
		framePtr = CMSMVirtualBoardArray[i];
		if (framePtr)
		{							/* Just look at this drivers boards 	*/
			if (CMSMstrcmp(	
							framePtr->ConfigTable.MLIDCFG_ShortName,
							CurrentConfigTable->MLIDCFG_ShortName) == 0)
			{
			 	switch (parameterType)
				{
					case INTPARAM:
						if (*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_Interrupt0)
							retValue = i;
						break;
					case INT1PARAM:
						if (*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_Interrupt1)
							retValue = i;
						break;
					case PORTPARAM:
						if (*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_IOPort0)
							retValue = i;
						break;
					case PORT1PARAM:
						if (*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_IOPort1)
							retValue = i;
						break;
					case DMAPARAM:
						if (*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_DMALine0)
							retValue = i;
						break;
					case DMA1PARAM:
						if (*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_DMALine1)
							retValue = i;
						break;
					case MEMPARAM:
						if ((void *)*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_MemoryAddress0)
							retValue = i;
						break;
					case MEM1PARAM:
						if ((void *)*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_MemoryAddress1)
							retValue = i;
						break;
					case SLOTPARAM:
						if (*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_Slot)
							retValue = i;
						break;
					case CHANNELPARAM:
						if (*(UINT32 *)value ==
								framePtr->ConfigTable.MLIDCFG_ChannelNumber)
							retValue = i;
						break;
				}
			}
		}
	}
	return (retValue);
}
#endif

/*======================================================================
  Name:	CMSMParseSingleParameter(
				struct _DRIVER_OPTION_	*driverOption)

  Description:		This function will parse for a parameter described by a
			driverOption structure returning the value found in the
			driverOption structure. This function handles only one
 			parameter and it should be custom parameter. This function
 			can be called multiple times in DriverInit


  Parameters in:	
		driverOption 		-	Ptr to a driver option structure;
 									for only one possible parameter.

  Values returned:		
	ODISTAT_SUCCESSFUL 	- Parameter Sucessfully Parsed
	ODISTAT_FAIL			- Failure in Parseing parameter
	ODISTAT_ITEM_NOT_PRESENT- Parse Item not present, not able to parse
	ODISTAT_BAD_PARAMETER	- Parameter Parsed not within range or value.

  Implied Parameters:

  Notes:				

=======================================================================*/
ODISTAT _cdecl
CMSMParseSingleParameter(
					struct _DRIVER_OPTION_	*driverOption)
{
	ODISTAT						status;
	UINT32						ParseType;
	MEON_STRING					SCnvSpecBuf[80];
	MEON_STRING					*Keyword, *CnvSpec;
	MEON_STRING					*SCnvSpec = SCnvSpecBuf;
	UINT16						origParamType;

	/* The ParsetString must be non null for parsing to continue. */
	if (driverOption->ParseString == NULL)
	{
		return ODISTAT_FAIL;
	}

	/* Save the original driver option type and set it to CUSTOMPARAM.  At */
	/* this point can only parser for custom parameters.  We do not want the */
	/* config table modified and there may not be a config table to modify. */
	origParamType = driverOption->Type;
	driverOption->Type = CUSTOMPARAM;

	if ((status = ParseDriverOption(
					driverOption,
					&ParseType,
					&Keyword,
					&CnvSpec,
					&SCnvSpec)) == ODISTAT_SUCCESSFUL )
	{
			status = ParseParam(
					driverOption,
					ParseType,
					Keyword,
					CnvSpec,
					SCnvSpec);
	}
	/* Restore original type */
	driverOption->Type = origParamType;
	return status;
}

/*======================================================================
  Name:		CMSMInitParser(
			DRIVER_PARM_BLOCK	*hsmParmBlock)

  Description:	This function will initialize the parser to ensure that
		the parser can identify new board instances
		from each other, in parsing, thus keeping
 		instances seperate and avoiding problems.
 		This routine should be called once per board
		instance, before any call to either parser
		procedure.

  Parameters In:
		hsmParmBlock      -	Ptr to The HSM Parameter Block.

  Returns:		
	ODISTAT_SUCCESSFUL 	- Currently this procedure does not fail, it is
				either called or not

  Notes:				

=======================================================================*/
ODISTAT _cdecl
CMSMInitParser(DRIVER_PARM_BLOCK	*hsmParmBlock)
{
#ifdef OS_NT
	UINT32 i;
#endif

   CommandLine = hsmParmBlock->DriverInitParmPointer->CommandLine;

#ifdef OS_NT
	/* The command line is single NULL separated per parameter. */
	/* Remove the single NULL separaters from the command line */
	/* and replace it with a space so that other APIs will see */
	/* a single null terminated string. */
   for (i = 0; *(UINT16 *)&CommandLine[i] != 0; ++i)
	{
      if (CommandLine[i] == '\0')
		{
         CommandLine[i] = ' ';
		}
	}
#else
	gBuffersBelow16 = 0;
#endif
	CMSMstrupr(CommandLine);
	parserInitialized = TRUE;	
	return ODISTAT_SUCCESSFUL;
}
/*=======================================================================*/
/*=======================================================================*/
